#================================================================#
# CLASSES
#================================================================#

import es
import os
from es import server_var
from random import randrange
from math import sqrt
from etmod_2008.includes.includes import *
from etmod_2008.logs.logs import *
from etmod_2008.languages.languages import *

#___________________________________________CLASSES

# ------------ #
# CLASSES #
# ------------ #
''' Used to represent actions done in a class and xp given for them '''
class Actions:
	def __init__(
		self,
		kill = 10,
		death = 0,
		suicide = 10,
		teamkill = 20,
		headshot = 5,
		knife = 5,
		hurt = 1,
		selfhurt = 1,
		teamhurt = 2,
		heal = 0,
		plant = 0,
		defuse = 0,
		explode = 0,
		supply = 0,
		rescue = 0,
		killMoney = 100,
		deathMoney = 100,
		suicideMoney = 100,
		teamKillMoney = 200,
		headshotMoney = 50,
		knifeMoney = 50,
		hurtMoney = 10,
		selfHurtMoney = 10,
		teamHurtMoney = 20,
		healMoney = 0,
		plantMoney = 0,
		defuseMoney = 0,
		explodeMoney = 0,
		supplyMoney = 0,
		rescueMoney = 0):
		self.KillXp = kill
		self.DeathXp = death
		self.SuicideXp = suicide
		self.TeamKillXp = teamkill
		self.HeadshotXp = headshot
		self.KnifeXp = knife
		self.HurtXp = hurt
		self.SelfHurtXp = selfhurt
		self.TeamHurtXp = teamhurt
		self.HealXp = heal
		self.PlantXp = plant
		self.DefuseXp = defuse
		self.ExplodeXp = explode
		self.SupplyXp = supply
		self.RescueXp = rescue
		self.KillMoney = killMoney
		self.DeathMoney = deathMoney
		self.SuicideMoney = suicideMoney
		self.TeamKillMoney = teamKillMoney
		self.HeadshotMoney = headshotMoney
		self.KnifeMoney = knifeMoney
		self.HurtMoney = hurtMoney
		self.SelfHurtMoney = selfHurtMoney
		self.TeamHurtMoney = teamHurtMoney
		self.HealMoney = healMoney
		self.PlantMoney = plantMoney
		self.DefuseMoney = defuseMoney
		self.ExplodeMoney = explodeMoney
		self.SupplyMoney = supplyMoney
		self.RescueMoney = rescueMoney
	def CanHeal(self):
		return self.HealXp > 0
	def CanPlant(self):
		return self.PlantXp > 0
	def CanDefuse(self):
		return self.DefuseXp > 0
	def CanExplode(self):
		return self.ExplodeXp > 0
	def CanSupply(self):
		return self.SupplyXp > 0
	def CanRescue(self):
		return self.RescueXp > 0
		
''' used to represent the stuff and prices that can be used to upgrade a class '''
class ClassUpgrades:
	def __init__(self):
		self.BetterWeaponT = None
		self.BetterWeaponCT = None
		self.MaxAmmoPrimaryT = 0
		self.MaxAmmoPrimaryCT = 0
		self.MaxAmmoSecondaryT = 0
		self.MaxAmmoSecondaryCT = 0
		self.MaxInvAmountT = 0
		self.MaxInvAmountCT = 0
		self.HealthPrice = 0
		self.ArmorPrice = 0
		self.SpeedPrice = 0
		self.PrimaryAmmoPriceT = 0
		self.PrimaryAmmoPriceCT = 0
		self.SecondaryAmmoPriceT = 0
		self.SecondaryAmmoPriceCT = 0
		self.InventoryPriceT = 0
		self.InventoryPriceCT = 0
		self.BetterWeaponPriceT = 0
		self.BetterWeaponPriceCT = 0
	def CurrentPrice(self, defPrice, level, curAmount):
		price = int(sqrt(float(defPrice) * int(level) * int(curAmount)))
		if price < defPrice: price = defPrice
		return price
		
''' Used to represent ETmod classes '''
class Class:
	def __init__(self, name):
		self.Name = name
		self.Active = 1
		self.UnlockClass = ""
		self.UnlockLevel = 0
		self.MaxAllowed = 100
		self.WeaponPrimaryT = None
		self.WeaponSecondaryT = None
		self.InventoryT = None
		self.AmmoPrimaryT = 0
		self.AmmoSecondaryT = 0
		self.ClipPrimaryT = 0
		self.ClipSecondaryT = 0
		self.InvAmountT = 0
		self.WeaponPrimaryCT = None
		self.WeaponSecondaryCT = None
		self.InventoryCT = None
		self.AmmoPrimaryCT = 0
		self.AmmoSecondaryCT = 0
		self.ClipPrimaryCT = 0
		self.ClipSecondaryCT = 0
		self.InvAmountCT = 0
		self.Health = 1
		self.Armor = 0
		self.Speed = 100
		self.Color = Color()
		self.Actions = Actions()
		self.Upgrades = ClassUpgrades()
	def LoadFromFile(self, filename = ""):
		f = None
		try:
			if filename == "":
				filename = self.Name
			fpath = os.path.join(os.getcwd(), clConfigPath)
			fpath = os.path.join(fpath, filename + ".cl")
			f = open(fpath, "r")
			for line in f:
				parts = str.split(line)
				if len(parts) == 2:
					var = str.strip(parts[0], "\"")
					val = str.strip(parts[1], "\"")
					self.SetValue(var, val)
		except:
			log("Error while loading class file: " + str(filename))
		finally:
			if f is not None:
				f.close()
	def SetValue(self, var, val):
		var = str.lower(var)
		
		if var == "active":
			log("Active: " + str(val))
			self.Active = int(val)
		elif var == "unlock_class":
			self.UnlockClass = val
		elif var == "unlock_level":
			self.UnlockLevel = int(val)
		elif var == "max_allowed":
			self.MaxAllowed = int(val)
		elif var == "weapon_primary_t":
			val = str(val).replace("weapon_", "")
			wp = weapons.get(val, None)
			if wp is not None and wp.IsPrimary():
				self.WeaponPrimaryT = wp
		elif var == "weapon_secondary_t":
			val = str(val).replace("weapon_", "")
			wp = weapons.get(val, None)
			if wp is not None and wp.IsSecondary():
				self.WeaponSecondaryT = wp
		elif var == "inventory_t":
			val = str(val).replace("weapon_", "")
			val = val.replace("item_", "")
			wp = weapons.get(val, None)
			if wp is not None and wp.IsInventory():
				self.InventoryT = wp
		elif var == "ammo_primary_t":
			self.AmmoPrimaryT = int(val)
		elif var == "ammo_secondary_t":
			self.AmmoSecondaryT = int(val)
		elif var == "clip_primary_t":
			self.ClipPrimaryT = int(val)
		elif var == "clip_secondary_t":
			self.ClipSecondaryT = int(val)
		elif var == "inv_amount_t":
			self.InvAmountT = int(val)
		elif var == "weapon_primary_ct":
			val = str(val).replace("weapon_", "")
			wp = weapons.get(val, None)
			if wp is not None and wp.IsPrimary():
				self.WeaponPrimaryCT = wp
		elif var == "weapon_secondary_ct":
			val = str(val).replace("weapon_", "")
			wp = weapons.get(val, None)
			if wp is not None and wp.IsSecondary():
				self.WeaponSecondaryCT = wp
		elif var == "inventory_ct":
			val = str(val).replace("weapon_", "")
			val = str(val).replace("item_", "")
			wp = weapons.get(val, None)
			if wp is not None and wp.IsInventory():
				self.InventoryCT = wp
		elif var == "ammo_primary_ct":
			self.AmmoPrimaryCT = int(val)
		elif var == "ammo_secondary_ct":
			self.AmmoSecondaryCT = int(val)
		elif var == "clip_primary_ct":
			self.ClipPrimaryCT = int(val)
		elif var == "clip_secondary_ct":
			self.ClipSecondaryCT = int(val)
		elif var == "inv_amount_ct":
			self.InvAmountCT = int(val)
		elif var == "health":
			self.Health = int(val)
		elif var == "armor":
			self.Armor = int(val)
		elif var == "color":
			parts = str.split(val, ";")
			self.Color.SetRed(int(parts[0]))
			self.Color.SetGreen(int(parts[1]))
			self.Color.SetBlue(int(parts[2]))
			self.Color.SetAlpha(int(parts[3]))
		elif var == "speed":
			self.Speed = int(val)
		elif var == "kill_xp":
			self.Actions.KillXp = int(val)
		elif var == "death_xp":
			self.Actions.DeathXp = int(val)
		elif var == "suicide_xp":
			self.Actions.SuicideXp = int(val)
		elif var == "teamkill_xp" or var == "tk_xp":
			self.Actions.TeamKillXp = int(val)
		elif var == "headshot_xp" or var == "hs_xp":
			self.Actions.HeadshotXp = int(val)
		elif var == "knife_xp" or var == "cut_xp":
			self.Actions.KnifeXp = int(val)
		elif var == "hurt_xp":
			self.Actions.HurtXp = int(val)
		elif var == "selfhurt_xp":
			self.Actions.SelfHurtXp = int(val)
		elif var == "teamhurt_xp":
			self.Actions.TeamHurtXp = int(val)
		elif var == "heal_xp":
			self.Actions.HealXp = int(val)
		elif var == "plant_xp":
			self.Actions.PlantXp = int(val)
		elif var == "defuse_xp":
			self.Actions.DefuseXp = int(val)
		elif var == "explode_xp":
			self.Actions.ExplodeXp = int(val)
		elif var == "supply_xp":
			self.Actions.SupplyXp = int(val)
		elif var == "rescue_xp":
			self.Actions.RescueXp = int(val)
		elif var == "kill_money":
			self.Actions.KillMoney = int(val)
		elif var == "death_money":
			self.Actions.DeathMoney = int(val)
		elif var == "suicide_money":
			self.Actions.SuicideMoney = int(val)
		elif var == "teamkill_money" or var == "tk_money":
			self.Actions.TeamKillMoney = int(val)
		elif var == "headshot_money":
			self.Actions.HeadshotMoney = int(val)
		elif var == "knife_money":
			self.Actions.KnifeMoney = int(val)
		elif var == "hurt_money":
			self.Actions.HurtMoney = int(val)
		elif var == "selfhurt_money":
			self.Actions.SelfHurtMoney = int(val)
		elif var == "teamhurt_money":
			self.Actions.TeamHurtMoney = int(val)
		elif var == "heal_money":
			self.Actions.HealMoney = int(val)
		elif var == "plant_money":
			self.Actions.PlantMoney = int(val)
		elif var == "defuse_money":
			self.Actions.DefuseMoney = int(val)
		elif var == "explode_money":
			self.Actions.ExplodeMoney = int(val)
		elif var == "supply_money":
			self.Actions.SupplyMoney = int(val)
		elif var == "rescue_money":
			self.Actions.RescueMoney = int(val)
		elif var == "better_weapon_t":
			val = str(val).replace("weapon_", "")
			wp = weapons.get(val, None)
			if wp is not None and wp.IsPrimary():
				self.Upgrades.BetterWeaponT = wp
		elif var == "better_weapon_ct":
			val = str(val).replace("weapon_", "")
			wp = weapons.get(val, None)
			if wp is not None and wp.IsPrimary():
				self.Upgrades.BetterWeaponCT = wp
		elif var == "max_ammo_primary_t":
			self.Upgrades.MaxAmmoPrimaryT = int(val)
		elif var == "max_ammo_primary_ct":
			self.Upgrades.MaxAmmoPrimaryCT = int(val)
		elif var == "max_ammo_secondary_t":
			self.Upgrades.MaxAmmoSecondaryT = int(val)
		elif var == "max_ammo_secondary_ct":
			self.Upgrades.MaxAmmoSecondaryCT = int(val)
		elif var == "max_inv_amount_t":
			self.Upgrades.MaxInvAmountT = int(val)
		elif var == "max_inv_amount_ct":
			self.Upgrades.MaxInvAmountCT = int(val)
		elif var == "hp_price" or var == "health_price":
			self.Upgrades.HealthPrice = int(val)
		elif var == "ap_price" or var == "armor_price":
			self.Upgrades.ArmorPrice = int(val)
		elif var == "speed_price":
			self.Upgrades.SpeedPrice = int(val)
		elif var == "prim_ammo_price_t" or var == "primary_ammo_price_t":
			self.Upgrades.PrimaryAmmoPriceT = int(val)
		elif var == "prim_ammo_price_ct" or var == "primary_ammo_price_ct":
			self.Upgrades.PrimaryAmmoPriceCT = int(val)
		elif var == "sec_ammo_price_t" or var == "secondary_ammo_price_t":
			self.Upgrades.SecondaryAmmoPriceT = int(val)
		elif var == "sec_ammo_price_ct" or var == "secondary_ammo_price_ct":
			self.Upgrades.SecondaryAmmoPriceCT = int(val)
		elif var == "inv_price_t" or var == "inventory_price_t":
			self.Upgrades.InventoryPriceT = int(val)
		elif var == "inv_price_ct" or var == "inventory_price_ct":
			self.Upgrades.InventoryPriceCT = int(val)
		elif var == "weapon_price_t" or var == "better_weapon_price_t":
			self.Upgrades.BetterWeaponPriceT = int(val)
		elif var == "weapon_price_ct" or var == "better_weapon_price_ct":
			self.Upgrades.BetterWeaponPriceCT = int(val)
	def __str__(self):
		key = server_var["etLogLanguage"]
		logLanguage = languages[str(key)]
		return\
			str.capitalize(logLanguage.Dict["words"]["name"]) + ": " + self.Name + "\n" +\
			str.capitalize(logLanguage.Dict["words"]["active"]) + ": " + str(self.Active) + "\n" +\
			str.capitalize(logLanguage.Dict["groups"]["unlock class"]) + ": " + str(self.UnlockClass) + "\n" +\
			str.capitalize(logLanguage.Dict["groups"]["unlock level"]) + ": " + str(self.UnlockLevel) + "\n" +\
			str.capitalize(logLanguage.Dict["groups"]["max allowed"]) + ": " + str(self.MaxAllowed) + "\n" +\
			str.capitalize(logLanguage.Dict["words"]["health"]) + ": " + str(self.Health) + "\n" +\
			str.capitalize(logLanguage.Dict["words"]["armor"]) + ": " + str(self.Armor) + "\n" +\
			str.capitalize(logLanguage.Dict["words"]["speed"]) + ": " + str(self.Speed) + "\n" +\
			str.capitalize(logLanguage.Dict["groups"]["prim weapons"]) + ": " +\
				"T=" + str(self.WeaponPrimaryT) + " - " +\
				"CT=" + str(self.WeaponPrimaryCT) + "\n" +\
			str.capitalize(logLanguage.Dict["groups"]["sec weapons"]) + ": " +\
				"T=" + str(self.WeaponSecondaryT) + " - " +\
				"CT=" + str(self.WeaponSecondaryCT) + "\n" +\
			str.capitalize(logLanguage.Dict["words"]["inventory"]) + ": " +\
				"T=" + str(self.InventoryT) + " - " +\
				"CT=" + str(self.InventoryCT) + "\n" +\
			str.capitalize(logLanguage.Dict["groups"]["prim ammo amount"]) + ": " +\
				"T=" + str(self.AmmoPrimaryT) + " - " +\
				"CT=" + str(self.AmmoPrimaryCT) + "\n" +\
			str.capitalize(logLanguage.Dict["groups"]["sec ammo amount"]) + ": " +\
				"T=" + str(self.AmmoSecondaryT) + " - " +\
				"CT=" + str(self.AmmoSecondaryCT) + "\n" +\
			str.capitalize(logLanguage.Dict["groups"]["inv amount"]) + ": " +\
				"T=" + str(self.InvAmountT) + " - " +\
				"CT=" + str(self.InvAmountCT) + "\n"
	def GiveStuff(self, player, team, protected = False):
		player = str(player)
		team = int(team)
		es.server.queuecmd("est_StripPlayer " + player)
		es.server.queuecmd("est_RestrictAll " + player)
		es.server.queuecmd("est_SetCash " + player + " 0")
		es.server.queuecmd("est_SetHealth " + player + " " + str(self.Health))
		es.server.queuecmd("est_SetArmor " + player + " " + str(self.Armor))
		es.server.queuecmd("est_Speed " + player + " " + str(float(self.Speed) / 100))
		es.server.queuecmd("est_UnRestrict " + player + " weapon_knife")
		es.server.queuecmd("est_Give " + player + " weapon_knife")
		es.server.queuecmd("est_God " + player + " " + str(int(protected)))
		if es.getInt("et_use_colors") == 1:
			es.server.queuecmd("est_SetPlayerColor " + player + " " +\
				str(self.Color.Red) + " " + str(self.Color.Green) + " " + str(self.Color.Blue) + " " + str(self.Color.Alpha) + " 1")
		else:
			es.server.queuecmd("est_SetPlayerColor " + player + " 255 255 255 255 0")
			
		#Allow all weapons of the same class
		if self.WeaponPrimaryT is not None:
			es.server.queuecmd("est_UnRestrict " + player + " " + self.WeaponPrimaryT.FullName())
		if self.WeaponSecondaryT is not None:
			es.server.queuecmd("est_UnRestrict " + player + " " + self.WeaponSecondaryT.FullName())
		if self.InventoryT is not None:
			es.server.queuecmd("est_UnRestrict " + player + " " + self.InventoryT.FullName())
		if self.WeaponPrimaryCT is not None:
			es.server.queuecmd("est_UnRestrict " + player + " " + self.WeaponPrimaryCT.FullName())
		if self.WeaponSecondaryCT is not None:
			es.server.queuecmd("est_UnRestrict " + player + " " + self.WeaponSecondaryCT.FullName())
		if self.InventoryT is not None:
			es.server.queuecmd("est_UnRestrict " + player + " " + self.InventoryCT.FullName())
			
		if team == 2:		# Terrorist
			# Primary weapon
			if self.WeaponPrimaryT is not None:
				es.server.queuecmd("est_Give " + player + " " + self.WeaponPrimaryT.FullName())
				self.WeaponPrimaryT.SetAmmo(player, int(self.AmmoPrimaryT))
				self.WeaponPrimaryT.SetClip(player, int(self.ClipPrimaryT))
			# Secondary weapon
			if self.WeaponSecondaryT is not None:
				es.server.queuecmd("est_Give " + player + " " + self.WeaponSecondaryT.FullName())
				self.WeaponSecondaryT.SetAmmo(player, int(self.AmmoSecondaryT))
				self.WeaponSecondaryT.SetClip(player, int(self.ClipSecondaryT))
			# Inventory
			if self.InventoryT is not None:
				es.server.queuecmd("es_setinfo x 0;es_setinfo y 0;es_setinfo z 0;es_getplayerlocation x y z " + player +\
					";es_math z + 20;es_setpos " + player + " server_var(x) server_var(y) server_var(z)")
				es.server.queuecmd("es_give " + player + " " + self.InventoryT.FullName())
		elif team == 3:		# Counter-Terrorist
			# Primary weapon
			if self.WeaponPrimaryCT is not None:
				es.server.queuecmd("est_Give " + player + " " + self.WeaponPrimaryCT.FullName())
				self.WeaponPrimaryCT.SetAmmo(player, int(self.AmmoPrimaryCT))
				self.WeaponPrimaryCT.SetClip(player, int(self.ClipPrimaryCT))
			# Secondary weapon
			if self.WeaponSecondaryCT is not None:
				es.server.queuecmd("est_Give " + player + " " + self.WeaponSecondaryCT.FullName())
				self.WeaponSecondaryCT.SetAmmo(player, int(self.AmmoSecondaryCT))
				self.WeaponSecondaryCT.SetClip(player, int(self.ClipSecondaryCT))
			# Inventory
			if self.InventoryT is not None:
				es.server.queuecmd("es_setinfo x 0;es_setinfo y 0;es_setinfo z 0;es_getplayerlocation x y z " + player +\
					";es_math z + 20;es_setpos " + player + " server_var(x) server_var(y) server_var(z)")
				es.server.queuecmd("es_give " + player + " " + self.InventoryCT.FullName())
				
#___________________________________________DECLARATIONS

# --------------------- #
# DECLARATIONS #
# --------------------- #
clConfigPath = "cstrike/addons/eventscripts/etmod_2008/classes/configuration"
classes = {}
classes["noclass"] = Class("noclass")

#___________________________________________FUNCTIONS

# ---------------- #
# FUNCTIONS #
# ---------------- #
''' Load ETmod class files '''
def loadClassFiles():
	curPath = os.path.join(os.getcwd(), clConfigPath)
	for fpath in os.listdir(curPath):
		log("Found a class file: " + fpath)
		tailParts = os.path.splitext(fpath)
		fileName = tailParts[0]
		ext = tailParts[1]
		if ext == ".cl":
			f = os.path.join(curPath, fpath)
			log("Class: " + fileName)
			log("File path: " + f)
			curCl = Class(fileName)
			classes[fileName] = curCl
			curCl.LoadFromFile()			
''' Get a random class '''
def getRandomClass():
	if len(classes.keys()) == 0:
		return None
	while 1:
		c = classes.keys()[randrange(len(classes.keys()))]
		if c != "noclass":
			return classes[c]

#___________________________________________SERVER EVENTS

# --------------------- #
# SERVER EVENTS #
# --------------------- #
''' Script loaded '''
def load():
	# Load files
	loadClassFiles()
	for c in classes.values():
		log("Class: " + c.Name)
			
	# Log
	log("Classes sub-script loaded")	
''' Script unloaded '''
def unload():
	# Log
	log("Classes sub-script unloaded")