#================================================================#
# INCLUDES
#================================================================#

import es
from es import server_var

#___________________________________________CLASSES

# ------------ #
# CLASSES #
# ------------ #
''' Used to represent ammo types '''
class AmmoType:
	def __init__(self, name, index, displayName = None):
		self.Name = name
		self.Index = index
		if displayName is None:
			displayName = name
		self.DisplayName = displayName
		global ammoTypes
		ammoTypes[name] = self
	def __str__(self):
		return self.DisplayName
''' Used to represent a weapon '''
class Weapon:
	def __init__(self, name, weaponClass, displayName = None, ammoType = None):
		self.Name = name
		self.Class = weaponClass
		self.AmmoType = ammoType
		if displayName is None:
			displayName = name
		self.DisplayName = displayName
		global weapons
		weapons[name] = self
	def IsPrimary(self):
		return\
			(self.Class == weaponClasses["shotgun"] or\
			self.Class == weaponClasses["smg"] or\
			self.Class == weaponClasses["rifle"] or\
			self.Class == weaponClasses["heavy"])
	def IsSecondary(self):
		return (self.Class == weaponClasses["pistol"])
	def IsInventory(self):
		return\
			(self.Class == weaponClasses["grenade"] or\
			self.Class == weaponClasses["bomb"] or\
			self.Class == weaponClasses["items"])
	def FullName(self):
		if self.Class == weaponClasses["items"]:
			return "item_" + self.Name
		else:
			return "weapon_" + self.Name
	def SetAmmo(self, pid, amount):
		# If it's an item, nothing to do
		if self.Class == weaponClasses["items"]:
			return
		# Check that amount is positive
		if amount < 0:
			amount = 0
		# Set player ammo
		#es.server.queuecmd("est_SetAmmo " + str(pid) + " " + self.Name + " " + str(amount))
		#es.setplayerprop(str(pid), propIndices["ammo"] + self.AmmoType.Index, amount)
		es.server.queuecmd("playerset ammo " + str(pid) + " " + self.Name + " " + str(amount))
	def SetClip(self, pid, amount):
		# If that's an item, nothing to do
		if self.Class == weaponClasses["items"]:
			return
		# Check that amount is positive
		if amount < 0:
			amount = 0
		# Set player clip
		es.server.queuecmd("playerset clip " + str(pid) + " " + self.Name + " " + str(amount))
		#es.setindexprop(self.AmmoType.Index, propIndices["clip"], amount)
	def __str__(self):
		return self.DisplayName
''' Used to represent a color '''
class Color:
	def __init__(self, red = 0, green = 0, blue = 0, alpha = 255):
		self.SetRed(red)
		self.SetGreen(green)
		self.SetBlue(blue)
		self.SetAlpha(alpha)
	
	def GetFromOther(self, other):
		self.SetRed(other.Red)
		self.SetGreen(other.Green)
		self.SetBlue(other.Blue)
		self.SetAlpha(other.Alpha)
	
	def SetColorBytes(self, color_bytes, value):
		val = int(value)
		if val < 0: val = 0
		elif val > 255: val = 255
		
		if color_bytes & colorBytes["red"]: self.Red = value
		if color_bytes & colorBytes["green"]: self.Green = value
		if color_bytes & colorBytes["blue"]: self.Blue = value
		if color_bytes & colorBytes["alpha"]: self.Alpha = value
		
	def SetRed(self, value):
		self.SetColorBytes(colorBytes["red"], value)
	def SetGreen(self, value):
		self.SetColorBytes(colorBytes["green"], value)
	def SetBlue(self, value):
		self.SetColorBytes(colorBytes["blue"], value)
	def SetAlpha(self, value):
		self.SetColorBytes(colorBytes["alpha"], value)
		
	def __str__(self):
		return str(self.Red) + "-"\
			+ str(self.Green) + "-"\
			+ str(self.Blue) + "-"\
			+ str(self.Alpha)
			
	def __int__(self):
		return int(self.Red + self.Green + self.Blue + self.Alpha)
		
#___________________________________________DECLARATIONS

# --------------------- #
# DECLARATIONS #
# --------------------- #
ammoTypes = {}
weapons = {}
weaponClasses = {
	"knife"		:	0,
	"pistol"	:	1,
	"shotgun"	:	2,
	"smg"		:	4,
	"rifle"		:	8,
	"heavy"		:	16,
	"grenade"	:	32,
	"bomb"		:	64,
	"items"		:	128
}
propIndices =	{
	"ammo"		:	"CCSPlayer.baseclass.localdata.m_iAmmo.",
	"clip"		:	"CBaseCombatWeapon.LocalWeaponData.m_iClip1",
	"health"	:	"CCSPlayer.baseclass.m_iHealth",
	"money"		:	"CCSPlayer.m_iAccount",
	"vecforce"	:	"CBaseEntity.m_vecForce",
	"position"	:	"CBaseEntity.m_vecOrigin",
	"velocityx"	:	"CBasePlayer.localdata.m_vecVelocity[0]",
	"velocityy"	:	"CBasePlayer.localdata.m_vecVelocity[1]",
	"velocityz"	:	"CBasePlayer.localdata.m_vecVelocity[2]"
}
colorBytes = {
	"red" :		1,
	"green"	:	2,
	"blue"	:	4,
	"alpha"	:	8
}

# Ammo types
Ammo50 = AmmoType("50", "001", ".50 Action Express")
Ammo556A = AmmoType("556A", "002", "5.56 Otan")
Ammo762 = AmmoType("762", "002", "7.62 Otan")
Ammo556B = AmmoType("556B", "003", "5.56 Otan")
Ammo223 = AmmoType("223", "003", ".223")
Ammo556C = AmmoType("556C", "004", "5.56 Parabellum")
Ammo338A = AmmoType("338A", "005", ".338 Lapua Magnum")
Ammo9para = AmmoType("9para", "006", "9mm Parabellum")
AmmoCal12 = AmmoType("cal12", "007", "Cal. 12")
Ammo45 = AmmoType("45", "008", ".45 ACP")
Ammo357 = AmmoType("357", "009", ".357 Sib")
Ammo57x28 = AmmoType("57x28", "010", "5.7x28 mm")
Ammo338B = AmmoType("338B", "010", ".338 Lapua Magnum")

# Weapons
Knife = Weapon("knife", weaponClasses["knife"], "Knife")
Usp = Weapon("usp", weaponClasses["pistol"], "Usp", ammoTypes["45"])
Glock = Weapon("glock", weaponClasses["pistol"], "Glock", ammoTypes["9para"])
P228 = Weapon("p228", weaponClasses["pistol"], "P228", ammoTypes["357"])
FiveSeven = Weapon("fiveseven", weaponClasses["pistol"], "Five-Seven", ammoTypes["57x28"])
Deagle = Weapon("deagle", weaponClasses["pistol"], "Desert Eagle", ammoTypes["50"])
Elite = Weapon("elite", weaponClasses["pistol"], "Dual Berretas", ammoTypes["9para"])
M3 = Weapon("m3", weaponClasses["shotgun"], "M3 Pump Shotgun", ammoTypes["cal12"])
XM1014 = Weapon("xm1014", weaponClasses["shotgun"], "XM1014 Auto Shotgun", ammoTypes["cal12"])
Tmp = Weapon("tmp", weaponClasses["smg"], "Steyr Tmp", ammoTypes["9para"])
Mac10 = Weapon("mac10", weaponClasses["smg"], "Mac 10", ammoTypes["45"])
Ump45 = Weapon("ump45", weaponClasses["smg"], "Ump 45", ammoTypes["45"])
Mp5Navy = Weapon("mp5navy", weaponClasses["smg"], "Mp5 Navy", ammoTypes["9para"])
P90 = Weapon("p90", weaponClasses["smg"], "FN P90", ammoTypes["338B"])
Galil = Weapon("galil", weaponClasses["rifle"], "Galil", ammoTypes["223"])
Famas = Weapon("famas", weaponClasses["rifle"], "Famas", ammoTypes["556B"])
M4A1 = Weapon("m4a1", weaponClasses["rifle"], "M16 AR15 A1", ammoTypes["556A"])
AK47 = Weapon("ak47", weaponClasses["rifle"], "Kalachnikov", ammoTypes["762"])
Aug = Weapon("aug", weaponClasses["rifle"], "Steyr Aug", ammoTypes["556A"])
Sg552 = Weapon("sg552", weaponClasses["rifle"], "Sig 552", ammoTypes["556B"])
Scout = Weapon("scout", weaponClasses["rifle"], "Steyr Scout", ammoTypes["762"])
Awp = Weapon("awp", weaponClasses["rifle"], "AWP Sniper Rifle", ammoTypes["338A"])
Sg550 = Weapon("sg550", weaponClasses["rifle"], "Sig 550 Auto Sniper", ammoTypes["556B"])
G3SG1 = Weapon("g3sg1", weaponClasses["rifle"], "G3SG1 Auto Sniper", ammoTypes["762"])
M249 = Weapon("m249", weaponClasses["heavy"], "M249", ammoTypes["556C"])
HEGrenade = Weapon("hegrenade", weaponClasses["grenade"], "Frag Grenade")
SmokeGrenade = Weapon("smokegrenade", weaponClasses["grenade"], "Smoke Grenade")
Flashbang = Weapon("flashbang", weaponClasses["grenade"], "Flashbang Grenade")
C4 = Weapon("c4", weaponClasses["bomb"], "C4")
defuser = Weapon("defuser", weaponClasses["items"], "Defuser")

#___________________________________________FUNCTIONS

# ---------------- #
# FUNCTIONS #
# ---------------- #
''' Get time string using ES without altering the current format '''
def getESTimeString(format = "%H:%M:%S"):
	old_format = server_var['eventscripts_timeformat']
	server_var['eventscripts_timeformat'] = format
	es.set("eventscripts_timeformat", format)
	es.set("ETtimestring", "None", "Latest time string")
	es.gettimestring("ETtimestring")	
	server_var['eventscripts_timeformat'] = old_format
	return server_var['ETtimestring']
	
''' Prevent someone from defusing a bomb '''
def preventFromDefusing(player):
	# Force them stop using their use key
	es.server.queuecmd("est_CExec " + str(player) + " -use")
	es.server.queuecmd("es_xset x 0;es_xset y 0;es_xset z 0; es est_GetViewAngle " + str(player) + " x y z;es_xmath y + 180;" +\
		"es est_SetViewAngle " + str(player) + " server_var(x) server_var(y) server_var(z)")
	es.setplayerprop(str(player), propIndices["velocityx"], 250)
	es.setplayerprop(str(player), propIndices["velocityy"], 250)
	es.setplayerprop(str(player), propIndices["velocityz"], 250)
	es.delayed(1, 'es_setplayerprop ' + str(player) + ' "' + propIndices["velocityx"] + '" 0')
	es.delayed(1, 'es_setplayerprop ' + str(player) + ' "' + propIndices["velocityy"] + '" 0')
	es.delayed(1, 'es_setplayerprop ' + str(player) + ' "' + propIndices["velocityz"] + '" 0')
	
''' Prevent someone from planting a bomb '''
def preventFromPlanting(player):
	# Force them stop using their attack key
	es.server.queuecmd("est_CExec " + str(player) + " -attack")
	es.server.queuecmd("est_RemoveWeapon " + str(player) + " weapon_c4")
	es.server.queuecmd("est_Give " + str(player) + " weapon_c4")

#___________________________________________SERVER EVENTS

# --------------------- #
# SERVER EVENTS #
# --------------------- #
''' Script loaded '''
def load():
	es.dbgmsg(0, "Includes sub-script loaded")
''' Script unloaded '''
def unload():
	es.dbgmsg(0, "Includes sub-script unloaded")