#================================================================#
# LANGUAGES
#================================================================#

import es
import os
from etmod_2008.includes.includes import *

#___________________________________________CLASSES

# ------------ #
# CLASSES #
# ------------ #
''' Used to represent ETmod language files '''
class Language:
	def __init__(self, name):
		self.Name = name
		self.Dict = {}
		self.Dict["logs"] = {}
		self.Dict["logs"]["includes ok"] = "Includes Ok"
		self.Dict["logs"]["logs ok"] = "Logs Ok"
		self.Dict["logs"]["languages ok"] = "Languages Ok"
		self.Dict["logs"]["classes ok"] = "Classes Ok"
		self.Dict["logs"]["stats ok"] = "Stats Ok"
		self.Dict["logs"]["main script ok"] = "Main script Ok"
		self.Dict["logs"]["Logging language updated to"] = "Logging language updated to"
		self.Dict["logs"]["Chosen logging language not found"] = "Chosen logging language not found"
		self.Dict["words"] = {}
		self.Dict["words"]["name"] = "name"
		self.Dict["words"]["active"] = "active"
		self.Dict["words"]["unlock"] = "unlock"
		self.Dict["words"]["class"] = "class"
		self.Dict["words"]["level"] = "level"
		self.Dict["words"]["health"] = "health"
		self.Dict["words"]["armor"] = "armor"
		self.Dict["words"]["speed"] = "speed"
		self.Dict["words"]["inventory"] = "inventory"
		self.Dict["groups"] = {}
		self.Dict["groups"]["unlock class"] = "unlock class"
		self.Dict["groups"]["unlock level"] = "unlock level"
		self.Dict["groups"]["max allowed"] = "max. allowed"
		self.Dict["groups"]["prim weapons"] = "prim. weapons"
		self.Dict["groups"]["sec weapons"] = "sec. weapons"
		self.Dict["groups"]["prim ammo amount"] = "prim. ammo amount"
		self.Dict["groups"]["sec ammo amount"] = "sec. ammo amount"
		self.Dict["groups"]["inv amount"] = "inv. amount"
	def LoadFromFile(self, filename = ""):
		f = None
		try:
			if filename == "":
				filename = self.Name
			fpath = os.path.join(os.getcwd(), lgConfigPath)
			fpath = os.path.join(fpath, filename + ".lg")
			es.dbgmsg(0, "File path: " + fpath)
			f = open(fpath, "r")
			cat = ""
			for line in f:
				parts = str.split(line, "\"")
				if len(parts) == 3:
					cat = parts[1]
					es.dbgmsg(0, "Cat: " + cat)
				elif len(parts) == 5:
					es.dbgmsg(0, parts[1] + " = " + parts[3])
					self.Dict[cat][parts[1]] = parts[3]
		except:
			es.dbgmsg(0, "Error while loading language file: " + str(filename))
		finally:
			if f is not None:
				f.close()
	def ContainsWord(self, word):
		return self.Dict["words"].has_key(word)
	def ContainsLog(self, log):
		return self.Dict["logs"].has_key(log)
	def ContainsGroup(self, group):
		return self.Dict["group"].has_key(group)
	def TranslateWord(self, word):
		if self.ContainsWord(word):
			return self.Dict["words"][word]
		return word
	def TranslateLog(self, log):
		if self.ContainsLog(log):
			return self.Dict["logs"][log]
		return log
	def TranslateGroup(self, group):
		if self.ContainsGroup(group):
			return self.Dict["groups"][group]
		return group
	def Translate(self, text,  checkGroups = True, checkLogs = True, checkWords = True):
		# Build the string without the escape sequences
#		str = text.replace("\\x01", "")
#		str = text.replace("\\x03", "")
#		str = text.replace("\\x04", "")
		
		# If the text should be cut, cut it
		translated = ""
		for part in text.split("%_"):
			translated += self.TranslatePart(part, checkGroups, checkLogs, checkWords)
			
		# Put back the escape sequences in the string
#		index = 0
#		while index < len(text) and (index = text.find("\\x01", index)) != -1:			
			
		# For escape sequences
		#text = text.replace("\\x01", "\x01")
		#text = text.replace("\\x03", "\x03")
		#text = text.replace("\\x04", "\x04")
		
		# Return - and treat possible escape sequences
		return translated.decode("string_escape")
	def TranslatePart(self, text, checkGroups = True, checkLogs = True, checkWords = True):
		# Groups
		if checkGroups:
			for k in self.Dict["groups"]:
				text = text.replace(k, self.Dict["groups"][k])
				text = text.replace(k.capitalize(), self.Dict["groups"][k].capitalize())
		# Logs
		if checkLogs:
			for k in self.Dict["logs"]:
				text = text.replace(k, self.Dict["logs"][k])
				text = text.replace(k.capitalize(), self.Dict["logs"][k].capitalize())
		# Words
		if checkWords:
			for k in self.Dict["words"]:
				# Make sure it's really a single word
				pos = text.lower().find(k.lower())
				if	pos != -1 and\
					((pos == 0 and (pos == len(text) - len(k) or not text[pos + len(k)].isalpha())) or\
					(pos == len(text) - len(k) and (pos == 0 or not text[pos - 1].isalpha())) or\
					(not text[pos - 1].isalpha() and  not text[pos + len(k)].isalpha())):
					text = text.replace(k, self.Dict["words"][k])
					text = text.replace(k.capitalize(), self.Dict["words"][k].capitalize())
					
		# Return
		return text
	def __str__(self):
		s = "Name: " + self.Name + "\n"
		for cat in self.Dict.keys():
			for var in self.Dict[cat].keys():
				s += cat + "." + var + " = " + self.Dict[cat][var] + "\n"
		return s
		
#___________________________________________DECLARATIONS

# --------------------- #
# DECLARATIONS #
# --------------------- #
lgConfigPath = "cstrike/addons/eventscripts/etmod_2008/languages/configuration"
languages = {}
languages["default"] = Language("default")

#___________________________________________FUNCTIONS

# ---------------- #
# FUNCTIONS #
# ---------------- #
''' Load ETmod language files '''
def loadLanguageFiles():
	curPath = os.path.join(os.getcwd(), lgConfigPath)
	es.dbgmsg(0, "Cur path: " + str(curPath))
	for fpath in os.listdir(curPath):
		es.dbgmsg(0, "Found a file: " + str(fpath))
		tailParts = os.path.splitext(fpath)
		fileName = tailParts[0]
		es.dbgmsg(0, "Filename: " + fileName)
		ext = tailParts[1]
		es.dbgmsg(0, "Extension: " + ext)
		if ext == ".lg":
			f = os.path.join(curPath, fpath)
			es.dbgmsg(0, "Language: " + fileName)
			es.dbgmsg(0, "File path: " + f)
			curLg = Language(fileName)
			languages[fileName] = curLg
			curLg.LoadFromFile()

#___________________________________________SERVER EVENTS

# --------------------- #
# SERVER EVENTS #
# --------------------- #
''' Script loaded '''
def load():
	# Load files
	loadLanguageFiles()
			
	# Log
	es.dbgmsg(0, "Languages sub-script loaded")
''' Script unloaded '''
def unload():
	es.dbgmsg(0, "Languages sub-script unloaded")