#================================================================#
# LOGS
#================================================================#

import es
from etmod_2008.includes.includes import *
from etmod_2008.languages.languages import *

#___________________________________________DECLARATIONS

# --------------------- #
# DECLARATIONS #
# --------------------- #
logTypes =	{
				"none"		:	0,
				"console"	:	1,
				"chat"		:	2,
				"file"		:	4
			}
logConfigPath = "cstrike/addons/eventscripts/etmod_2008/logs/configuration"
logDataPath = "cstrike/addons/eventscripts/etmod_2008/logs/data"

#___________________________________________FUNCTIONS

# ---------------- #
# FUNCTIONS #
# ---------------- #
''' Use a default configuration '''
def useDefaultConfig():
	es.set("etLogLanguage", "default", "Language used for logging")
	es.set("etLogPrefix", "[ETmod - %H:%M:%S] ", "Prefix displayed before of log lines")
	es.set("etFileNameFormat", "%m%d%y", "Format of the log file names")
	es.set("etLogTo", logTypes["console"], "Where to display log lines. Could include console(1), chat(2) and file(4) used as flags.")
	log("Log config reset")
''' Load logging configuration from config file '''
def loadConfig():
	f = None
	try:
		f = open(os.path.join(logConfigPath, "config.cfg"))
		for line in f:
			parts = line.split("=")
			if len(parts) == 2:
				parts[1] = parts[1].strip("\n")
				parts[1] = parts[1].strip("\r")
				# Log language
				if parts[0] == "language":
					if not languages.has_key(parts[1]):
						log("Logging language not found: " + parts[1])
						es.set("etLogLanguage", "default")
					else:
						log("Logging language changed to " + parts[1])
						es.set("etLogLanguage", parts[1])
					#es.set("etLogLanguage", "default")
					#for l in languages.values():
					#	if parts[1].find(l.Name) != -1:
					#		es.set("etLogLanguage", l.Name)
					#		log("Logging language updated to " + l.Name)
					#		break
				# Log prefix
				elif parts[0] == "prefix":
					log("New log prefix: " + parts[1])
					es.set("etLogPrefix", parts[1])
				# Log file name format
				elif parts[0] == "file_name_format":
					log("New log file name format: " + parts[1])
					es.set("etLogFileNameFormat", parts[1])
				# Log to
				elif parts[0] == "log_to":
					targets = str.split(parts[1], ";")
					logInto = logTypes["none"]
					for target in targets:
						if logTypes.has_key(target):
							log("Now logging to " + target)
							logInto |= logTypes[target]						
					es.set("etLogTo", logInto)
	except:
		log("Couldn't load logging configuration from config file")
	finally:
		if f is not None:
			f.close()
'''
Log a message into console / chat / file
depending on the selected log types
'''
def log(msg):
	# Retreive the time string
	timeStr = getESTimeString(server_var["etLogPrefix"])
	
	# Prepare the message
	language = str(server_var["etLogLanguage"])
	if languages.has_key(language):
		l = languages[language]
		msg = l.Translate(msg, False, True, False)
	
	# Output the message to the different logging targets
	types = server_var["etLogTo"]
	msg = timeStr + msg
	if types & logTypes["console"]:
		es.dbgmsg(0, msg)
	if types & logTypes["chat"]:
		es.msg(msg)
	if types & logTypes["file"]:
		f = None
		try:
			fpath = os.path.join(os.getcwd(), os.path.join(logDataPath, getESTimeString(server_var["etLogFileNameFormat"]) + ".log"))
			f = open(fpath, "a")			
			f.write(msg + "\r\n")
		except:
			es.dbgmsg(0, "Couldn't write log message to log file")
		finally:
			if f is not None:
				f.close()

#___________________________________________SERVER EVENTS

# --------------------- #
# SERVER EVENTS #
# --------------------- #
''' Script loaded '''
def load():
	# Load logs config
	es.dbgmsg(0, "Languages from logs: ")
	for k in languages.keys():
		es.dbgmsg(0, k)
	useDefaultConfig()
	loadConfig()
	
	# Log
	log("Logs sub-script loaded")
''' Script unloaded '''
def unload():
	# Log
	log("Logs sub-script unloaded")