#================================================================#
# PLAYERS
#================================================================#

import es, os, playerlib, gamethread
from es import server_var
from etmod_2008.includes.includes import *
from etmod_2008.languages.languages import languages
from etmod_2008.logs.logs import log
from etmod_2008.skills.skills import *
from etmod_2008.classes.classes import classes
from etmod_2008.stats.stats import Stats
from etmod_2008.shop.shop import *

#___________________________________________CLASSES

# ------------ #
# CLASSES #
# ------------ #
''' Used to represent a players' configuration for the mod '''
class PlayerConfig:
	def __init__(self):
		self.Language = languages["default"]
		self.InGameInfo = True
		self.UseNewWeapon = True
	def Save(self, filename, playerName = ""):
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(plPrefPath, filename + ".cfg")), "w")
			if playerName != "":
				f.write("# " + playerName + "'s preferences file\n\n")
			f.write("language=" + self.Language.Name + "\n")
			f.write("in_game_info=" + str(int(self.InGameInfo)) + "\n")
			f.write("use_new_weapon=" + str(int(self.UseNewWeapon)) + "\n")
			log("'" + filename + "' preferences file successfully saved")
		except:
			log("Couldn't write player preferences to file " + filename)
		finally:
			if f is not None:
				f.close()
	def Load(self, filename):
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(plPrefPath, filename + ".cfg")), "r")
			for line in f:
				parts = str.split(line, "=")
				if len(parts) == 2:
					parts[1] = parts[1].strip("\n")
					parts[1] = parts[1].strip("\r")
					if parts[0] == "language":
						self.Language = languages[parts[1]]
					elif parts[0] == "in_game_info":
						self.InGameInfo = (int(parts[1]) == 1)
					elif parts[0] == "use_new_weapon":
						self.UseNewWeapon = (int(parts[1]) == 1)
			log("'" + str(filename) + "' preferences file successfully loaded")
		except:
			log("Couldn't load player preferences from file " + filename)
		finally:
			if f is not None:
				f.close()
		
''' Used to represent a player's situation while playing '''
class PlayerSituation:
	def __init__(
		self,
		alive = False,
		invAmount = 0,
		page = 1,
		health = 1,
		planted = 0):
		self.Alive = alive
		self.InvAmount = invAmount
		self.Page = page
		self.Planted = planted
		self.Selection = ""
		self.SkillInUse = None
		self.SkillValue = None
		self.SufferingFrom = []
		self.TakingProfitFrom = []
		
''' Used to represent a skill in progression for a player '''
class PlayerSkill:
	def __init__(self, player, skill):
		self.Player = player
		self.Skill = skill
		self.Amount = 0
		self.Progression = {}
		for p in skill.Parameters.values():
			self.Progression[p.Type] = p.Default
	def Use(self):
		# If he doesn't have one, warn him
		if self.Amount < 1:
			es.tell(self.Player.ID, "You don't have any instance of that skill currently. Buy some to use it.")
			return
			
		# If he's currently using another skill, warn him
		if	self.Player.Situation.SkillInUse is not None and\
			isinstance(self.Player.Situation.SkillInUse, PlayerSkill) and\
			self.Player.Situation.SkillValue is None:
			es.tell(self.Player.ID, "You're currently using skill '" + self.Player.Situation.SkillInUse.Skill.Name + "'")
			return
		
		# Invisibility
		if self.Skill == skills["invisibility"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			intensity = self.Progression[skillParameterTypes["intensity"]]
			
			# Get class color
			color = Color()
			color.GetFromOther(self.Player.Class.Color)
			color.SetAlpha(255 - intensity)
			
			# Use class color if 'et_use_colors' is activated
			cc = es.getInt("et_use_colors")
			if cc == 0:
				color.SetRed(255)
				color.SetGreen(255)
				color.SetBlue(255)
				
			# Set player transparency
			self.Player.Situation.SkillInUse = self
			es.server.queuecmd("est_SetPlayerColor " + str(self.Player.ID) + " " + str(color.Red) +\
				" " + str(color.Green) + " " + str(color.Blue) + " " + str(color.Alpha) + " 1")
				
			# Decrease amount
			self.Amount -= 1
				
			# Info message
			es.tell(self.Player.ID, "Invisibility on for " + str(duration) + " seconds")
			
			# Reset after delay
			#es.delayed(duration, "est_SetPlayerColor " + str(self.Player.ID) + " " + str(color.Red) +\
			#	" " + str(color.Green) + " " + str(color.Blue) + " " + str(color.Alpha) + " 1")
			gamethread.delayed(duration, self.Deactivate, ())
			
		# Shield
		elif self.Skill == skills["shield"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			hp = self.Progression[skillParameterTypes["health"]]
			ap = self.Progression[skillParameterTypes["armor"]]
			
			# Increase player health and armor
			self.Player.Situation.SkillInUse = self
			es.server.queuecmd("est_SetHealth " + str(self.Player.ID) + " " + str(self.Player.Class.Health + hp))
			es.server.queuecmd("est_SetArmor " + str(self.Player.ID) + " " + str(self.Player.Class.Armor + ap))
			
			# Decrease amount
			self.Amount -= 1
			
			# Info message
			es.tell(self.Player.ID, "Shield on for " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, ())
			
		# Invincibility
		elif self.Skill == skills["invincibility"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			
			# Set player god like
			self.Player.Situation.SkillInUse = self
			es.server.queuecmd("est_God " + str(self.Player.ID) + " 1")
			
			# Decrease amount
			self.Amount -= 1
			
			# Info message
			es.tell(self.Player.ID, "Invincibility on for " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, ())
			
		# Poison
		elif self.Skill == skills["poison"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			hp = self.Progression[skillParameterTypes["health"]]
			ap = self.Progression[skillParameterTypes["armor"]]
			speed = self.Progression[skillParameterTypes["speed"]]
			
			# Get victim
			es.server.queuecmd("es_set et_view_player 0;est_getviewplayer " + str(self.Player.ID) + " et_view_player")
			victim = int(server_var["et_view_player"])
			
			# Decrease amount even if he fails
			self.Amount -= 1
			
			# If he doesn't have a victim in sight, tell him he used one skill for nothing
			if victim == 0:
				es.tell(self.Player.ID, "You failed giving poison to an enemy")
				return
			# If he was aiming at a friend, tell him he cannot poison his friends
			if players[str(victim)].Team == self.Player.Team:
				es.tell(self.Player.ID, "Don't try to poison your teammates !")
				return
			# If he's got an enemy in sight, poison him
			# Update player and enemy's situation
			self.Player.Situation.SkillInUse = self
			players[str(victim)].Handicap(self)
			
			# Info message
			es.tell(self.Player.ID, players[str(victim)].Name + " will suffer from your poison for " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, (victim))
			
		# Boots
		elif self.Skill == skills["boots"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			speed = self.Progression[skillParameterTypes["speed"]]
			cspeed = self.Player.Class.Speed
			
			# Increase players' speed
			self.Player.Situation.SkillInUse = self
			es.server.queuecmd("est_Speed " + str(self.Player.ID) + " " + str(float(cspeed + speed) / 100))
			
			# Decrease amount
			self.Amount -= 1
			
			# Info message
			es.tell(self.Player.ID, "Boots on for " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, ())
			
		# Vulnerability
		elif self.Skill == skills["vulnerability"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			hp = self.Progression[skillParameterTypes["health"]]
			ap = self.Progression[skillParameterTypes["armor"]]
			
			# Get victim
			es.server.queuecmd("es_set et_view_player 0;est_getviewplayer " + str(self.Player.ID) + " et_view_player")
			victim = int(server_var["et_view_player"])
			
			# Decrease amount even if he fails
			self.Amount -= 1
			
			# If he doesn't have a victim in sight, tell him he used one skill for nothing
			if victim == 0:
				es.tell(self.Player.ID, "You failed giving vulnerability to an enemy")
				return
			# If he was aiming at a friend, tell him he cannot poison his friends
			if players[str(victim)].Team == self.Player.Team:
				es.tell(self.Player.ID, "Don't try this on your teammates !")
				return
			
			# Update player and enemy's situation
			self.Player.Situation.SkillInUse = self
			players[str(victim)].Situation.SufferingFrom.append(self)
			
			# Info message
			es.tell(self.Player.ID, players[str(victim)].Name + " will get more damage from his enemies for " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, (victim))
			
		# Armor piercing
		elif self.Skill == skills["armorpiercing"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			hp = self.Progression[skillParameterTypes["health"]]
			ap = self.Progression[skillParameterTypes["armor"]]
			
			# Get victim
			es.server.queuecmd("es_set et_view_player 0;est_getviewplayer " + str(self.Player.ID) + " et_view_player")
			victim = int(server_var["et_view_player"])
			
			# Update player and enemy's situation
			self.Player.Situation.SkillInUse = self
			players[str(victim)].Situation.SufferingFrom.append(self)
			
			# Decrease amount
			self.Amount -= 1
			
			# Info message
			es.tell(self.Player.ID, players[str(victim)].Name + " will get more damage from you for " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, (victim))
			
		# Grenades
		elif self.Skill == skills["grenades"]:
			# Get current info
			team = self.Player.Team			
			inventory = None
			if team == 2: inventory = self.Player.Class.InventoryT
			elif team == 3: inventory = self.Player.Class.InventoryCT
			
			# Only if his class uses grenades
			if inventory is not None and inventory.Class == weaponClasses["grenade"]:
				# Get player's current amount
				curAmount = self.Player.Situation.InvAmount
				
				# Increase and give him
				curAmount += 1
				self.Player.Situation.InvAmount = curAmount
				
				# Decrease amount
				self.Amount -= 1
				
				# Info message
				es.tell(self.Player.ID, "You received one more '" + inventory.DisplayName + "'")
				
		# Fast heal
		elif self.Skill == skills["fastheal"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			hp = self.Progression[skillParameterTypes["health"]]
			ap = self.Progression[skillParameterTypes["armor"]]
			
			# Update player's situation
			self.Player.Situation.SkillInUse = self			
			
			# Decrease amount
			self.Amount -= 1
			
			# Info message
			es.tell(self.Player.ID, "You will heal with " + str(hp) + " more HP and " + str(ap) + " more AP during " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, ())
			
		# Fast supply
		elif self.Skill == skills["fastsupply"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			primary = self.Progression[skillParameterTypes["primary"]]
			secondary = self.Progression[skillParameterTypes["secondary"]]
			
			# Update player's situation
			self.Player.Situation.SkillInUse = self
			
			# Decrease amount
			self.Amount -= 1
			
			# Info message
			es.tell(self.Player.ID, "You will supply with " + str(primary) + " more primary ammo and " +\
				str(secondary) + " more secondary ammo during " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, ())
			
		# Jetpack
		elif self.Skill == skills["jetpack"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			
			# Update player's situation
			self.Player.Situation.SkillInUse = self
			
			# Activate the jetpack
			es.server.queuecmd("est_Jetpack " + str(self.Player.ID) + " 1")
			
			# Decrease amount
			self.Amount -= 1
			
			# Info message
			es.tell(self.Player.ID, "You'll be using your jetpack during " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, ())
			
		# Teleport
		elif self.Skill == skills["teleport"]:			
			# If the location has already been chosen, teleport player if not too far away from location
			if self.Player.Situation.SkillInUse is not None:
				# Get current info
				distance = self.Progression[skillParameterTypes["distance"]]
				loc = self.Player.Situation.SkillInUse
				p = playerlib.getPlayer(self.Player.ID)
				x = p.attributes["x"]
				y = p.attributes["y"]
				z = p.attributes["z"]
				
				# Check if the location is too far
				if	abs(loc[0] - x) > distance or\
					abs(loc[1] - y) > distance or\
					abs(loc[2] - z) > distance:
					es.tell(self.Player.ID, "You're too far from your teleport location")
				# Otherwise, teleport him
				else:
					# Teleport
					p.set("location", loc)
					
					# Decrease amount
					self.Amount -= 1
					
					# Info message
					es.tell(self.Player.ID, "You successfully teleported yourself")
					
					# Reset teleport location
					self.Player.Situation.SkillInUse = None
				
			# Otherwise, prepare the teleport location
			else:
				# Get current location
				p = playerlib.getPlayer(self.Player.ID)
				x = p.attributes["x"]
				y = p.attributes["y"]
				z = p.attributes["z"]
				
				# Update player's situation
				self.Player.Situation.SkillInUse = (x, y, z)
				
				# Info message
				es.tell(self.Player.ID, "Teleport location chosen")
				
		# Health pack
		elif self.Skill == skills["healthpack"]:
			# Get current info
			hp = self.Progression[skillParameterTypes["health"]]
			ap = self.Progression[skillParameterTypes["armor"]]
			player = self.Player
			c = player.Class
			s = player.Stats
			
			# Get victim
			es.server.queuecmd("es_set et_view_player 0;est_getviewplayer " + str(self.Player.ID) + " et_view_player")
			teammate = int(server_var["et_view_player"])
			
			# Decrease amount even if he fails
			self.Amount -= 1
			
			# If he doesn't have a victim in sight, tell him he used one skill for nothing	-	commented to allow giving to oneself
			#if teammate == 0:
			#	es.tell(self.Player.ID, "You failed giving a health pack to a teammate")
			#	return
			# To allow giving to oneself	-	uncommented to allow giving to oneself
			if teammate == 0:
				teammate = self.Player.ID
			# If he was aiming at an enemy, tell him he can't
			if players[str(teammate)].Team != self.Player.Team:
				es.tell(self.Player.ID, "This would be ridiculous to heal your enemies !")
				return
				
			# Get teammate's current health and armor
			p = playerlib.getPlayer(teammate)
			health = p.attributes["health"]
			armor = p.attributes["armor"]
			chealth = players[str(teammate)].Class.Health
			carmor = players[str(teammate)].Class.Armor
				
			# Boost teammate's health and armor
			newHealth = health + hp
			newArmor = armor + ap
			if newHealth > chealth: newHealth = chealth
			if newArmor > carmor: newArmor = carmor
			es.server.queuecmd("est_SetHealth " + str(teammate) + " " + str(newHealth))
			es.server.queuecmd("est_SetArmor " + str(teammate) + " " + str(newArmor))
			
			# Give xp and money to player for healing a teammate
			if teammate != self.Player.ID:
				self.Player.Stats.Update(self.Player.Class.Name, self.Player.Class.Name, "heals", (newHealth - health), s.GetLevel(c.Name))
				self.Player.Stats.Update(self.Player.Class.Name, self.Player.Class.Name, "heals", (newArmor - armor), s.GetLevel(c.Name))
			
			# Info message
			if players[str(teammate)] == self.Player:
				es.tell(self.Player.ID, "You successfully gave a health pack to yourself")
			else:
				es.tell(self.Player.ID, "You successfully gave a health pack to " + players[str(teammate)].Name)
			
		# Ammo pack
		elif self.Skill == skills["ammopack"]:
			# Get current info
			primary = self.Progression[skillParameterTypes["primary"]]
			secondary = self.Progression[skillParameterTypes["secondary"]]
			player = self.Player
			c = player.Class
			s = player.Stats
			
			# Get victim
			es.server.queuecmd("es_set et_view_player 0;est_getviewplayer " + str(player.ID) + " et_view_player")
			teammate = int(server_var["et_view_player"])
			
			# Decrease amount even if he fails
			self.Amount -= 1
			
			# If he doesn't have a victim in sight, tell him he used one skill for nothing	-	commented to allow giving to oneself
			#if teammate == 0:
			#	es.tell(self.Player.ID, "You failed giving an ammo pack to a teammate")
			#	return
			# To allow giving to oneself	-	uncommented to allow giving to oneself
			if teammate == 0:
				teammate = player.ID
			# If he was aiming at an enemy, tell him he can't
			if players[str(teammate)].Team != player.Team:
				es.tell(player.ID, "This would be ridiculous to supply your enemies !")
				return
				
			# Get teammate's current info
			p = playerlib.getPlayer(teammate)
			primAmmo = p.get("ammo", "primary")
			secAmmo = p.get("ammo", "secondary")
			team = players[str(teammate)].Team
			cPrimAmmo = 0
			cSecAmmo = 0
			if team == 2:
				cPrimAmmo = players[str(teammate)].Class.AmmoPrimaryT
				cSecAmmo = players[str(teammate)].Class.AmmoSecondaryT
			elif team == 3:
				cPrimAmmo = players[str(teammate)].Class.AmmoPrimaryCT
				cSecAmmo = players[str(teammate)].Class.AmmoSecondaryCT
				
			# Boost teammate's primary and secondary ammo
			newPrimAmmo = primAmmo + primary
			newSecAmmo = secAmmo + secondary
			if newPrimAmmo > cPrimAmmo: newPrimAmmo = cPrimAmmo
			if newSecAmmo > cSecAmmo: newSecAmmo = cSecAmmo
			p.set("ammo", ("primary", newPrimAmmo))
			p.set("ammo", ("secondary", newSecAmmo))
			
			# Give xp and money to player for supplying a teammate
			if teammate != player.ID:
				s.Update(c.Name, c.Name, "supplies", (newPrimAmmo - primAmmo), s.GetLevel(c.Name))
				s.Update(c.Name, c.Name, "supplies", (newSecAmmo - secAmmo), s.GetLevel(c.Name))
			
			# Info message
			if players[str(teammate)] == player:
				es.tell(player.ID, "You successfully gave an ammo pack to yourself")
			else:
				es.tell(player.ID, "You successfully gave an ammo pack to " + players[str(teammate)].Name)
				
		# Remote charges
		elif self.Skill == skills["remotecharges"]:
			# Get some info
			player = self.Player
			id = player.ID
				
			# If the location has already been chosen, trigger the explosion
			if player.Situation.SkillInUse is not None:
				# Get current info
				intensity = self.Progression[skillParameterTypes["intensity"]]
				duration = self.Progression[skillParameterTypes["duration"]]
				c = player.Class
				s = player.Stats
				loc = player.Situation.SkillInUse
				x = loc[0]
				y = loc[1]
				z = loc[2]
				
				# Prepare a variable so that killed player can reward the charge owner
				es.set("et_charge_owner_hurt", player.ID)
				es.set("et_charge_owner_kill", player.ID)
				
				# Trigger the charge explosion
				createExplosion(id, intensity, loc)
				es.setinfo("et_tmp_player", "0")
				es.server.queuecmd('es est_NearCoord et_tmp_player #all 500 ' + str(x) + ' ' + str(y) + ' ' + str(z) +\
					' "es est_Fade server_var(et_tmp_player) 0 ' + str(duration) + ' 1 255 255 255 255"')
				
				# Decrease amount
				self.Amount -= 1
				
				# Info message
				es.tell(self.Player.ID, "You successfully blew up your charge")
				
				# Reset charge location
				player.Situation.SkillInUse = None
				
			# Otherwise, prepare the charge location
			else:
				# Get current location
				p = playerlib.getPlayer(id)
				x = p.attributes["x"]
				y = p.attributes["y"]
				z = p.attributes["z"]
				
				# Update player's situation
				self.Player.Situation.SkillInUse = (x, y, z)
				
				# Info message
				es.tell(id, "Charge location chosen")
			
		# Binoculars
		elif self.Skill == skills["binoculars"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			distance = self.Progression[skillParameterTypes["distance"]]
			player = self.Player
			
		# Napalm grenades
		elif self.Skill == skills["napalmgrenades"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			hp = self.Progression[skillParameterTypes["health"]]
			ap = self.Progression[skillParameterTypes["armor"]]
			speed = self.Progression[skillParameterTypes["speed"]]
			distance = self.Progression[skillParameterTypes["distance"]]
			player = self.Player
			c = player.Class
			s = player.Stats
			
		# Exploding bullets
		elif self.Skill == skills["explodingbullets"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			player = self.Player
			
			# Set the skill in use
			self.Player.Situation.SkillInUse = self
			
			# Decrease amount
			self.Amount -= 1
			
			# Info message
			es.tell(self.Player.ID, "Exploding bullets on for " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, ())
			
		# Mini sticky bombs
		elif self.Skill == skills["ministickybombs"]:
			# Get some info
			player = self.Player
			id = player.ID
				
			# If the location has already been chosen, trigger the explosion
			if player.Situation.SkillInUse is not None:
				# Get current info
				intensity = self.Progression[skillParameterTypes["intensity"]]
				duration = self.Progression[skillParameterTypes["duration"]]
				locs = player.Situation.SkillValue
				
				# Trigger a chain reaction for the explosions
				chainedExplosions(id, intensity, locs, float(5) / float(duration))
				
				# Decrease amount
				self.Amount -= 1
				
				# Info message
				es.tell(self.Player.ID, "You successfully blew up your sticky bombs")
				
				# Reset value and deactivate skill
				player.Situation.SkillInUse = None
				player.Situation.SkillValue = None
				
			# Otherwise, prepare the list
			else:
				# Update player's situation
				player.Situation.SkillInUse = self
				player.Situation.SkillValue = []
				
				# Info message
				es.tell(id, "Sticky bombs ready to be used. Shoot them where you want.")
			
	def Deactivate(self, victim = 0):
		# Get info
		skill = self.Player.Situation.SkillInUse
		
		# If he's using this one then deactivate it
		if skill == self:
			# Reset player's situation
			self.Player.Situation.SkillInUse = None
			
			# For invisibility, reset player color
			if skill.Skill.Name == "invisibility":
				# Get class color
				color = Color()
				color.GetFromOther(self.Player.Class.Color)
				color.SetAlpha(255)
				
				# Use class color if 'et_use_colors' is activated
				cc = es.getInt("et_use_colors")
				if cc == 0:
					color.SetRed(255)
					color.SetGreen(255)
					color.SetBlue(255)
					
				# Reset player color
				es.server.queuecmd("est_SetPlayerColor " + str(self.Player.ID) + " " + str(color.Red) +\
					" " + str(color.Green) + " " + str(color.Blue) + " " + str(color.Alpha) + " 1")
					
				# Info message
				es.tell(self.Player.ID, "Invisibility deactivated")
				
			# For shield, reset player health / armor
			elif skill.Skill.Name == "shield":
				# Reset player's health / armor
				es.server.queuecmd("est_SetHealth " + str(self.Player.ID) + " " + str(self.Player.Class.Health))
				es.server.queuecmd("est_SetArmor " + str(self.Player.ID) + " " + str(self.Player.Class.Armor))
				
				# Info message
				es.tell(self.Player.ID, "Shield deactivated")
				
			# For invincibility, reset player god mode
			elif skill.Skill.Name == "invincibility":
				# Reset player back to normal
				es.server.queuecmd("est_God " + str(self.Player.ID) + " 0")
				
				# Info message
				es.tell(self.Player.ID, "Invincibility deactivated")
				
			# For poison, reset victim's situation
			elif skill.Skill.Name == "poison":
				# Reset victim's situation
				players[str(victim)].Situation.SufferingFrom.remove(skill)
				
				# Info message
				es.tell(self.Player.ID, players[str(victim)].Name + " is no more suffering from your poison")
				
			# For speed, reset the class value
			elif skill.Skill.Name == "boots":
				#Reset player's speed
				es.server.queuecmd("est_Speed " + str(self.Player.ID) + " " + str(float(self.Player.Class.Speed) / 100))
				
				# Info message
				es.tell(self.Player.ID, "Boots deactivated")
				
			# Vulnerability
			elif skill.Skill.Name == "vulnerability":
				# Reset victim's situation
				players[str(victim)].Situation.SufferingFrom.remove(skill)
				
				# Info message
				es.tell(self.Player.ID, "Vulnerability deactivated")
				
			# Armor piercing
			elif skill.Skill.Name == "armorpiercing":
				# Reset victim's situation
				players[str(victim)].Situation.SufferingFrom.remove(skill)
				
				# Info message
				es.tell(self.Player.ID, "Armor piercing deactivated")
				
			# Fast heal
			elif skill.Skill.Name == "fastheal":
				# Info message
				es.tell(self.Player.ID, "Fast heal deactivated")
				
			# Fast supply
			elif skill.Skill.Name == "fastsupply":
				# Info message
				es.tell(self.Player.ID, "Fast supply deactivated")
				
			# Jetpack
			elif skill.Skill.Name == "jetpack":
				# Deactivate the jetpack
				es.server.queuecmd("est_Jetpack " + str(self.Player.ID) + " 0")
				
				# Info message
				es.tell(self.Player.ID, "Jetpack deactivated")
				
			# Exploding bullets
			elif skill.Skill.Name == "explodingbullets":
				# Info message
				es.tell(self.Player.ID, "Exploding bullets deactivated")
		
''' Used to represent ETmod players '''
class Player:
	def __init__(
		self,
		index = -1,
		id = -1,
		steamid = "STEAM_ID_UNDEFINED",
		name = "unamed",
		team = 0,
		curclass = classes["noclass"]):
		self.Index = index
		self.ID = id
		self.SteamID = steamid
		self.Name = name
		self.Team = team
		self.Class = curclass
		self.NextClass = None
		self.Preferences = PlayerConfig()
		self.Stats = Stats()
		self.Skills = {}
		self.Situation = PlayerSituation()
		self.BoughtStuff = BoughtStuff()
	def IsBot(self):
		return self.SteamID == "BOT"
	def IsPending(self):
		return self.SteamID == "STEAM_ID_PENDING"
	def IsUndefined(self):
		return self.SteamID == "STEAM_ID_UNDEFINED"
	def GiveClassStuff(self, protected = False):
		if self.Team == 2 and self.Class.InventoryT is not None:
			self.Situation.InvAmount = self.Class.InvAmountT + self.BoughtStuff.Classes[self.Class.Name]["inv_amount_t"]
		elif self.Team == 3 and self.Class.InventoryCT is not None:
			self.Situation.InvAmount = self.Class.InvAmountCT + self.BoughtStuff.Classes[self.Class.Name]["inv_amount_ct"]
		self.Class.GiveStuff(self.ID, self.Team, protected)
		self.BoughtStuff.GiveStuff(self.ID, self.Team, self.Class.Name, self.Preferences.UseNewWeapon)
	def Heal(self, responsible = None):
		# Get info
		#health = es.getplayerprop(self.ID, propIndices["health"])
		p = playerlib.getPlayer(self.ID)
		health = p.attributes["health"]
		armor = p.attributes["armor"]
		
		# Update info
		newHealth = health + 1
		newArmor = armor
		if responsible is not None and responsible.Situation.SkillInUse is not None and responsible.Situation.SkillInUse.Skill.Name == "fastheal":
			newHealth += responsible.Situation.SkillInUse.Progression[skillParameterTypes["health"]]
			newArmor += responsible.Situation.SkillInUse.Progression[skillParameterTypes["armor"]]
			
		# Ensure new info is valid
		if newHealth > (self.Class.Health + self.BoughtStuff.Classes[self.Class.Name]["health"]):
			newHealth = self.Class.Health
		if newArmor > (self.Class.Armor + self.BoughtStuff.Classes[self.Class.Name]["armor"]):
			newArmor = self.Class.Armor
			
		# Use new info
		es.server.queuecmd("est_SetHealth " + str(self.ID) + " " + str(newHealth))
		es.server.queuecmd("est_SetArmor " + str(self.ID) + " " + str(newArmor))
		
		#return (newHealth != health)
		return (newHealth - health)
	def Supply(self, responsible = None):
		# PlayerLib stuff
		player = playerlib.getPlayer(self.ID)
		primAmmo = player.get("ammo", "primary")
		secAmmo = player.get("ammo", "secondary")
		
		# New amount
		newPrimAmmo = primAmmo + 1
		newSecAmmo = secAmmo + 1
		if responsible is not None and responsible.Situation.SkillInUse is not None and responsible.Situation.SkillInUse.Skill.Name == "fastsupply":
			newPrimAmmo += responsible.Situation.SkillInUse.Progression[skillParameterTypes["primary"]]
			newSecAmmo += responsible.Situation.SkillInUse.Progression[skillParameterTypes["secondary"]]
		
		# Set to max if too much
		if self.Team == 2:
			if newPrimAmmo > (self.Class.AmmoPrimaryT + self.BoughtStuff.Classes[self.Class.Name]["primary_ammo_t"]):
				newPrimAmmo = self.Class.AmmoPrimaryT
			if newSecAmmo > (self.Class.AmmoSecondaryT + self.BoughtStuff.Classes[self.Class.Name]["secondary_ammo_t"]):
				newSecAmmo = self.Class.AmmoSecondaryT
		elif self.Team == 3:
			if newPrimAmmo > (self.Class.AmmoPrimaryCT + self.BoughtStuff.Classes[self.Class.Name]["primary_ammo_ct"]):
				newPrimAmmo = self.Class.AmmoPrimaryCT
			if newSecAmmo > (self.Class.AmmoSecondaryCT + self.BoughtStuff.Classes[self.Class.Name]["secondary_ammo_ct"]):
				newSecAmmo = self.Class.AmmoSecondaryCT
				
		# Set new ammo
		player.set("ammo", ("primary", newPrimAmmo))
		player.set("ammo", ("secondary", newSecAmmo))
		
		# Give inventory item if none left
		invAmount = self.Situation.InvAmount
		if invAmount == 0:
			if self.Team == 2:
				if self.Class.InventoryT is not None:
					es.server.queuecmd("est_Give " + str(self.ID) + " " + self.Class.InventoryT.FullName())
			elif self.Team == 3:
				if self.Class.InventoryCT is not None:
					es.server.queuecmd("est_Give " + str(self.ID) + " " + self.Class.InventoryCT.FullName())
					
		# Increment count
		newInvAmount = invAmount + 1
		
		# Set to max if greater
		if self.Team == 2 and newInvAmount > (self.Class.InvAmountT + self.BoughtStuff.Classes[self.Class.Name]["inv_amount_t"]):
			newInvAmount = (self.Class.InvAmountT + self.BoughtStuff.Classes[self.Class.Name]["inv_amount_t"])
		elif self.Team == 3 and newInvAmount > (self.Class.InvAmountCT + self.BoughtStuff.Classes[self.Class.Name]["inv_amount_ct"]):
			newInvAmount = (self.Class.InvAmountCT + self.BoughtStuff.Classes[self.Class.Name]["inv_amount_ct"])
			
		# Set to player situation
		self.Situation.InvAmount = newInvAmount
			
		# Return a value to precise if player ammo was updated
		return ((newPrimAmmo - primAmmo) + (newSecAmmo - secAmmo) + (newInvAmount - invAmount))
		#return\
		#	(newPrimAmmo > primAmmo) or\
		#	(newSecAmmo > secAmmo) or\
		#	(newInvAmount > invAmount)
	def Handicap(self, pskill):
		# Get info
		l = self.Preferences.Language
		
		# Update his situation
		self.Situation.SufferingFrom.append(pskill)
		
		# Tell him
		es.tell(self.ID, l.Translate("You've been handicaped by " + pskill.Player.Name + " with skill '" + pskill.Skill.Name + "'"))
		
		# Start the loop
		gamethread.delayed(0, self.HandicapLoop(pskill))
	def HandicapLoop(self, pskill):
		# Get global info
		l = self.Preferences.Language
		speed = pskill.Progression[skillParameterTypes["speed"]]
		
		# If player's situation has been updated and he's no more suffering, tell him and leave
		if not self.Situation.SufferingFrom:
			es.tell(self.ID, l.Translate("You're not more suffering from '" + pskill.Skill.Name + "'"))
			return
		
		# Poison
		if pskill.Skill.Name == "poison":
			# Get info
			p = playerlib.getPlayer(self.ID)
			health = p.attributes["health"]#es.getplayerprop(self.ID, propIndices["health"])
			armor = p.attributes["armor"]
			hp = pskill.Progression[skillParameterTypes["health"]]
			ap = pskill.Progression[skillParameterTypes["armor"]]
			
			# Reduce his health and armor if he's alive
			if not p.attributes["isdead"]:
				armor -= ap
				if armor < 0: armor = 0			
				es.server.queuecmd("est_SetArmor " + str(self.ID) + " " + str(armor))
				health -= hp
				if health <= 0: es.server.queuecmd("est_Slay " + str(self.ID))
				else : es.server.queuecmd("est_SetHealth " + str(self.ID) + " " + str(health))
			
		# If his situation hasn't changed, call it back based on the skill's speed
		gamethread.delayed(10. / speed, self.HandicapLoop, (pskill))
	def SkillsDamage(self, attacker = None):
		# Check skills he's suffering from
		for pskill in self.Situation.SufferingFrom:
			# Vulnerability
			if pskill.Skill.Name == "vulnerability":
				# Get info
				p = playerlib.getPlayer(self.ID)
				health = p.attributes["health"]
				armor = p.attributes["armor"]
				hp = pskill.Progression[skillParameterTypes["health"]]
				ap = pskill.Progression[skillParameterTypes["armor"]]
				
				# Reduce his health and armor if he's alive
				if not p.attributes["isdead"]:
					armor -= ap
					if armor < 0: armor = 0
					es.server.queuecmd("est_SetArmor " + str(self.ID) + " " + str(armor))
					health -= hp
					if health <= 0: es.server.queuecmd("est_Slay " + str(self.ID))
					else : es.server.queuecmd("est_SetHealth " + str(self.ID) + " " + str(health))
				
			# Armor piercing
			elif pskill.Skill.Name == "armorpiercing" and attacker is not None:
				# Get info
				p = playerlib.getPlayer(self.ID)
				health = p.attributes["health"]
				armor = p.attributes["armor"]
				hp = pskill.Progression[skillParameterTypes["health"]]
				ap = pskill.Progression[skillParameterTypes["armor"]]
				
				# Reduce his health and armor if he's alive
				if not p.attributes["isdead"]:
					armor -= ap
					if armor < 0: armor = 0
					es.server.queuecmd("est_SetArmor " + str(self.ID) + " " + str(armor))
					health -= hp
					if health <= 0: es.server.queuecmd("est_Slay " + str(self.ID))
					else : es.server.queuecmd("est_SetHealth " + str(self.ID) + " " + str(health))
				
	def GetPlayerFileName(self):
		return str.replace(self.SteamID, ":", "_")
	def SaveStats(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		self.Stats.Save(self.GetPlayerFileName(), self.Name)
	def LoadStats(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		self.Stats.Load(self.GetPlayerFileName())
	def SavePreferences(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		self.Preferences.Save(self.GetPlayerFileName(), self.Name)
	def LoadPreferences(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		self.Preferences.Load(self.GetPlayerFileName())
	def SaveSkills(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(skillsDataPath, self.GetPlayerFileName() + ".skl")), "w")
			f.write("# " + self.Name + "'s skills file\n\n")
			for ps in self.Skills.values():
				f.write("[" + ps.Skill.Name + "]\n")
				f.write("amount=" + str(ps.Amount) + "\n")
				for p in ps.Skill.Parameters.keys():
					f.write(p + "=" + str(ps.Progression.get(skillParameterTypes[p], 0)) + "\n")
			log("'" + self.GetPlayerFileName() + "' skills file successfully saved")
		except:
			log("Error while trying to save skills for player " + self.Name)
		finally:
			if f is not None:
				f.close()
	def LoadSkills(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(skillsDataPath, self.GetPlayerFileName() + ".skl")), "r")
			self.Skills = {}
			skill = None
			pskill = None
			for line in f:
				line = line.strip("\n")
				line = line.strip("\r")
				if line.startswith("[") and line.endswith("]"):
					skillName = line.strip("[]")
					if skills.has_key(skillName):
						skill = skills[skillName]
						self.Skills[skillName] = PlayerSkill(self, skill)
						pskill = self.Skills[skillName]
					else:
						log("Skill not found for " + self.SteamID + ": " + skillName)
				elif pskill is not None:
					parts = line.split("=")
					if len(parts) == 2:
						var = parts[0]
						val = parts[1]
						if var == "amount":
							pskill.Amount = int(val)
						else:
							pskill.Progression[skillParameterTypes[var]] = int(val)
			log("'" + self.GetPlayerFileName() + "' skills file successfully loaded")
		except:
			log("Error while trying to load skills for player " + self.Name)
		finally:
			if f is not None:
				f.close()
	def SaveBoughtStuff(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		self.BoughtStuff.Save(self.GetPlayerFileName())
	def LoadBoughtStuff(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		self.BoughtStuff.Load(self.GetPlayerFileName())
	def __str__(self):
		return self.Name
		
#___________________________________________DECLARATIONS

# --------------------- #
# DECLARATIONS #
# --------------------- #
plPrefPath = "cstrike/addons/eventscripts/etmod_2008/players/preferences"
players = {}

#___________________________________________FUNCTIONS

# ---------------- #
# FUNCTIONS #
# ---------------- #
''' Player joining a class '''
def joinClass(player, targetClass, force = False):
	p = player
	c = targetClass
	
	# If done by an admin or needs to be forced for any reason, force hil
	if force:
		p.Class = c
		p.GiveClassStuff()
		return

	# Check if the chosen class is active
	if c.Active:
		# Check if he can already use that class
		if	es.getInt("et_restrict_classes") == 0 or\
			hasUnlockedClass(p, c):		
			# Check if there are not and will not be already enough players in that class
			if	es.getInt("et_restrict_players_in_classes") == 0 or\
				(isClassNotFull(c, p.Team) and willClassNotBeFull(c, p.Team)):
				# Check that player hasn't planted a bomb
				if p.Situation.Planted == 0:
					# Check if player doesn't yet have a class
					if p.Class == classes["noclass"]:
						p.Class = c
						p.GiveClassStuff()
					# Prepare his future class if he already has one
					else:
						p.NextClass = c
						es.tell(p.ID, "Next time you'll respawn as a " + c.Name)
				# Otherwise tell him that he can't change class				
				else:
					es.tell(p.ID, "You cannot change class when you have planted a bomb")
			# Otherwise tell him
			elif not (isClassNotFull(c, p.Team) and willClassNotBeFull(c, p.Team)):
				es.tell(p.ID, "That class is currently full. Please choose another one.")
		# Otherwise tell him he has to reach a certain level in a certain class first
		elif not hasUnlockedClass(p, c):
			es.tell(p.ID, "You must reach level " +\
				str(c.UnlockLevel) + " in class " + str(c.UnlockClass) +\
				" before you can play as a " + c.Name)
	# Otherwise tell him that the chosen class is not available
	else:
		es.tell(p.ID, "That class is not available currently")
		
#___________________________________________HELPER FUNCTIONS

# --------------------------- #
# HELPER FUNCTIONS #
# --------------------------- #
''' Check if player has what is required to join a class '''
def hasUnlockedClass(player, targetClass):
	# Check if player has the required level in the unlock class
	c = targetClass
	p = player
	return	c.UnlockClass == "noclass" or\
			c.UnlockClass == "" or\
			(classes.has_key(c.UnlockClass) and\
			p.Stats.Classes.has_key(c.UnlockClass) and\
			p.Stats.GetLevel(c.UnlockClass) >= int(c.UnlockLevel))			
''' Check if there's not too many people in a class already '''
def isClassNotFull(targetClass, team):
	c = targetClass
	
	# Player can join any class if he's alone on server
	if len(players) == 1:
		return True
	
	# Get how many players are in that team and how many are in that class
	inTeam = 0
	inClass = 0
	for p in players.values():
		if p.Team == team:
			inTeam += 1
			if p.Class == c:
				inClass += 1
	
	# Calculate how many people are allowed in that class		
	allowed = float(c.MaxAllowed) / 100 * inTeam
	
	# Return False if there will be too many with him
	return (inClass + 1) <= allowed		
''' Check if there will not be too many people in a class '''
def willClassNotBeFull(targetClass, team):
	c = targetClass
	
	# Get how many players are in that team and how many are in that class
	inTeam = 0
	inClass = 0
	for p in players.values():
		if p.Team == team:
			inTeam += 1
			if p.NextClass == c:
				inClass += 1
	
	# Calculate how many people are allowed in that class		
	allowed = float(c.MaxAllowed) / 100 * inTeam
	
	# Return False if there will be too many with him
	return (inClass + 1) <= allowed
	
#___________________________________________SERVER EVENTS

# --------------------- #
# SERVER EVENTS #
# --------------------- #
''' Script loaded '''
def load():
	# Log
	log("Players sub-script loaded")	
''' Script unloaded '''
def unload():
	# Log
	log("Players sub-script unloaded")