#================================================================#
# SERVER
#================================================================#

import es
import os
from etmod_2008.logs.logs import log

#___________________________________________CLASSES

# ------------ #
# CLASSES #
# ------------ #
''' Used to represent a server configuration for ETmod '''
class ServerConfig:
	def __init__(self, name):
		self.Name = name
		self.HostName = "- [ETmod2008 by McFly] -"
		self.AutoTeamBalance = True
		self.FriendlyFire = True
		self.BotQuota = 0
		self.BotQuotaMode = "normal"
		self.BotJoinAfterPlayer = 0
		self.BotJoinTeam = "any"
		self.BotJoinDelay = 0
		self.C4Timer = 45
		self.ChatTime = 10
		self.FadeToBlack = False
		self.FlashLight = True
		self.FootSteps = True
		self.FreezeTime = 3
		self.LimitTeams = 2
		self.MaxRounds = 0
		self.RoundTime = 5
		self.TimeLimit = 0
		self.TkPunish = False
		self.WinLimit = 0
	def GetCurrent(self):
		self.HostName = es.getString("hostname")
		self.AutoTeamBalance = (es.getInt("mp_autoteambalance") == 1)
		self.FriendlyFire = (es.getInt("mp_friendlyfire") == 1)
		self.BotQuota = es.getInt("bot_quota")
		self.BotQuotaMode = es.getString("bot_quota_mode")
		self.BotJoinAfterPlayer = (es.getInt("bot_join_after_player") == 1)
		self.BotJoinTeam = es.getString("bot_join_team")
		self.BotJoinDelay = es.getInt("bot_join_delay")
		self.C4Timer = es.getInt("mp_c4timer")
		self.ChatTime = es.getInt("mp_chattime")
		self.FadeToBlack = (es.getInt("mp_fadetoblack") == 1)
		self.FlashLight = (es.getInt("mp_flashlight") == 1)
		self.FootSteps = (es.getInt("mp_footsteps") == 1)
		self.FreezeTime = es.getInt("mp_freezetime")
		self.LimitTeams = es.getInt("mp_limitteams")
		self.MaxRounds = es.getInt("mp_maxrounds")
		self.RoundTime = es.getInt("mp_roundtime")
		self.TimeLimit = es.getInt("mp_timelimit")
		self.TkPunish = (es.getInt("mp_tkpunish") == 1)
		self.WinLimit = es.getInt("mp_winlimit")
		log("Retreived current server config into '" + self.Name + "' instance")
	def GetFromOther(self, other):
		self.HostName = other.HostName
		self.AutoTeamBalance = other.AutoTeamBalance
		self.FriendlyFire = other.FriendlyFire
		self.BotQuota = other.BotQuota
		self.BotQuotaMode = other.BotQuotaMode
		self.BotJoinAfterPlayer = other.BotJoinAfterPlayer
		self.BotJoinTeam = other.BotJoinTeam
		self.BotJoinDelay = other.BotJoinDelay
		self.C4Timer = other.C4Timer
		self.ChatTime = other.ChatTime
		self.FadeToBlack = other.FadeToBlack
		self.FlashLight = other.FlashLight
		self.FootSteps = other.FootSteps
		self.FreezeTime = other.FreezeTime
		self.LimitTeams = other.LimitTeams
		self.MaxRounds = other.MaxRounds
		self.RoundTime = other.RoundTime
		self.TimeLimit = other.TimeLimit
		self.TkPunish = other.TkPunish
		self.WinLimit = other.WinLimit
		log("Retreived server config from '" + other.Name + "' into '" + self.Name + "' instance")
	def GetFromText(self, text):
		for line in text:
			parts = str.split(line, "=")
			if len(parts) == 2:
				parts[1] = parts[1].strip("\n")
				parts[1] = parts[1].strip("\r")
				if parts[0] == "hostname":
					self.HostName = parts[1]
				elif parts[0] == "mp_autoteambalance":
					self.AutoTeamBalance = (int(parts[1]) == 1)
				elif parts[0] == "mp_friendlyfire":
					self.FriendlyFire = (int(parts[1]) == 1)
				elif parts[0] == "bot_quota":
					self.BotQuota = int(parts[1])
				elif parts[0] == "bot_quota_mode":
					self.BotQuotaMode = parts[1]
				elif parts[0] == "bot_join_after_player":
					self.BotJoinAfterPlayer = (int(parts[1]) == 1)
				elif parts[0] == "bot_join_team":
					self.BotJoinTeam = parts[1]
				elif parts[0] == "bot_join_delay":
					self.BotJoinDelay = int(parts[1])
				elif parts[0] == "mp_c4timer":
					self.C4Timer = int(parts[1])
				elif parts[0] == "mp_chattime":
					self.ChatTime = int(parts[1])
				elif parts[0] == "mp_fadetoblack":
					self.FadeToBlack = (int(parts[1]) == 1)
				elif parts[0] == "mp_flashlight":
					self.FlashLight = (int(parts[1]) == 1)
				elif parts[0] == "mp_footsteps":
					self.FootSteps = (int(parts[1]) == 1)
				elif parts[0] == "mp_freezetime":
					self.FreezeTime = int(parts[1])
				elif parts[0] == "mp_maxrounds":
					self.MaxRounds = int(parts[1])
				elif parts[0] == "mp_roundtime":
					self.RoundTime = int(parts[1])
				elif parts[0] == "mp_timelimit":
					self.TimeLimit = int(parts[1])
				elif parts[0] == "mp_tkpunish":
					self.TkPunish = (int(parts[1]) == 1)
				elif parts[0] == "mp_winlimit":
					self.WinLimit = int(parts[1])
	def LoadFromFile(self, filename = ""):
		if filename == "":
			filename = self.Name
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(serverConfigPath, filename + ".cfg")), "r")
			self.GetFromText(f)
			self.Name = filename
			log("'" + str(filename) + "' server config file successfully loaded")
		except:
			log("Server config file not found for '" + str(filename))
			# If 'et_noconfig_map_behaviour' is set to 'keep' then get current config to make sure it will not be changed
			if es.getString("et_noconfig_map_behaviour") == "keep":
				log("Using current config")
				self.GetCurrent()
			else:
				log("Using default config")
		finally:
			if f is not None:
				f.close()
	def SaveToFile(self, filename = ""):
		if filename == "":
			filename = self.Name
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(serverConfigPath, filename + ".cfg")), "w")
			f.write(str(self))
			log("'" + str(filename) + "' server config file successfully saved")
		except:
			log("Error while trying to save '" + str(filename) + "' server config file")
		finally:
			if f is not None:
				f.close()
	def Use(self):
		es.set("hostname", self.HostName)
		es.set("mp_autoteambalance", int(self.AutoTeamBalance))
		es.set("mp_friendlyfire", int(self.FriendlyFire))
		es.set("bot_quota", self.BotQuota)
		es.set("bot_quota_mode", self.BotQuotaMode)
		es.set("bot_join_after_player", int(self.BotJoinAfterPlayer))
		es.set("bot_join_team", self.BotJoinTeam)
		es.set("bot_join_delay", self.BotJoinDelay)
		es.set("mp_c4timer", self.C4Timer)
		es.set("mp_chattime", self.ChatTime)
		es.set("mp_fadetoblack", int(self.FadeToBlack))
		es.set("mp_flashlight", int(self.FlashLight))
		es.set("mp_footsteps", int(self.FootSteps))
		es.set("mp_freezetime", self.FreezeTime)
		es.set("mp_maxrounds", self.MaxRounds)
		es.set("mp_roundtime", self.RoundTime)
		es.set("mp_timelimit", self.TimeLimit)
		es.set("mp_tkpunish", int(self.TkPunish))
		es.set("mp_winlimit", self.WinLimit)
		log("'" + self.Name + "' server config now in use")
	def __str__(self):
		return\
			"hostname=" + self.HostName + "\n" +\
			"mp_autoteambalance=" + str(int(self.AutoTeamBalance)) + "\n" +\
			"mp_friendlyfire=" + str(int(self.FriendlyFire)) + "\n" +\
			"bot_quota=" + str(self.BotQuota) + "\n" +\
			"bot_quota_mode=" + self.BotQuotaMode + "\n" +\
			"bot_join_after_player=" + str(int(self.BotJoinAfterPlayer)) + "\n" +\
			"bot_join_team=" + self.BotJoinTeam + "\n" +\
			"bot_join_delay=" + str(self.BotJoinDelay) + "\n" +\
			"mp_c4timer=" + str(self.C4Timer) + "\n" +\
			"mp_chattime=" + str(self.ChatTime) + "\n" +\
			"mp_fadetoblack=" + str(int(self.FadeToBlack)) + "\n" +\
			"mp_flashlight=" + str(int(self.FlashLight)) + "\n" +\
			"mp_footsteps=" + str(int(self.FootSteps)) + "\n" +\
			"mp_freezetime=" + str(self.FreezeTime) + "\n" +\
			"mp_maxrounds=" + str(self.MaxRounds) + "\n" +\
			"mp_roundtime=" + str(self.RoundTime) + "\n" +\
			"mp_timelimit=" + str(self.TimeLimit) + "\n" +\
			"mp_tkpunish=" + str(int(self.TkPunish)) + "\n" +\
			"mp_winlimit="+ str(self.WinLimit)	
''' Used to represent a configuration for the mod '''
class ModConfig:
	def __init__(self, name):
		self.Name = name
		self.RestrictClasses = True
		self.RestrictPlayersInClasses = True
		self.UnlimitedAmmo = False
		self.NoReload = False
		self.SpawnDelay = 5
		self.SpawnProtectionTime = 4
		self.SynchronizedSpawn = False
		self.RandomClasses = False
		self.UseColors = False
		self.RoundXp = 50
		self.RoundMoney = 500
		self.HurtLoseXp = False
		self.NoConfigMapBehaviour = "keep"
	def GetCurrent(self):
		self.RestrictClasses = (es.getInt("et_restrict_classes") == 1)
		self.RestrictPlayersInClasses = (es.getInt("et_restrict_players_in_classes") == 1)
		self.UnlimitedAmmo = (es.getInt("et_unlimited_ammo") == 1)
		self.NoReload = (es.getInt("et_no_reload") == 1)
		self.SpawnDelay = es.getInt("et_spawn_delay")
		self.SpawnProtectionTime = es.getInt("et_spawn_protection_time")
		self.SynchronizedSpawn = (es.getInt("et_synchronized_spawn") == 1)
		self.RandomClasses = (es.getInt("et_random_classes") == 1)
		self.UseColors = (es.getInt("et_use_colors") == 1)
		self.RoundXp = es.getInt("et_round_xp")
		self.RoundMoney = es.getInt("et_round_money")
		self.HurtLoseXp = (es.getInt("et_hurt_lose_xp") == 1)
		self.NoConfigMapBehaviour = es.getString("et_noconfig_map_behaviour")
		log("Retreived current mod config into '" + self.Name + "' instance")
	def GetFromOther(self):
		self.RestrictClasses = other.RestrictClasses
		self.RestrictPlayersInClasses = other.RestrictPlayersInClasses
		self.UnlimitedAmmo = other.UnlimitedAmmo
		self.NoReload = other.NoReload
		self.SpawnDelay = other.SpawnDelay
		self.SpawnProtectionTime = other.SpawnProtectionTime
		self.SynchronizedSpawn = other.SynchronizedSpawn
		self.RandomClasses = other.RandomClasses
		self.UseColors = other.UseColors
		self.RoundXp = other.RoundXp
		self.RoundMoney = other.RoundMoney
		self.HurtLoseXp = other.HurtLoseXp
		self.NoConfigMapBehaviour = other.NoConfigMapBehaviour
		log("Retreived mod config from '" + other.Name + "' into '" + self.Name + "' instance")
	def GetFromText(self, text):
		for line in text:
			parts = str.split(line, "=")
			if len(parts) == 2:
				if parts[0] == "et_restrict_classes":
					self.RestrictClasses = (int(parts[1]) == 1)
				elif parts[0] == "et_restrict_players_in_classes":
					self.RestrictPlayersInClasses = (int(parts[1]) == 1)
				elif parts[0] == "et_unlimited_ammo":
					self.UnlimitedAmmo = (int(parts[1]) == 1)
				elif parts[0] == "et_no_reload":
					self.NoReload = (int(parts[1]) == 1)
				elif parts[0] == "et_spawn_delay":
					self.SpawnDelay = int(parts[1])
				elif parts[0] == "et_spawn_protection_time":
					self.SpawnProtectionTime = int(parts[1])
				elif parts[0] == "et_synchronized_spawn":
					self.SynchronizedSpawn = (int(parts[1]) == 1)
				elif parts[0] == "et_random_classes":
					self.RandomClasses = (int(parts[1]) == 1)
				elif parts[0] == "et_use_colors":
					self.UseColors = (int(parts[1]) == 1)
				elif parts[0] == "et_round_xp":
					self.RoundXp = int(parts[1])
				elif parts[0] == "et_round_money":
					self.RoundMoney = int(parts[1])
				elif parts[0] == "et_hurt_lose_xp":
					self.HurtLoseXp = (int(parts[1]) == 1)
				elif parts[0] == "et_noconfig_map_behaviour":
					self.NoConfigMapBehaviour = parts[1]
	def LoadFromFile(self, filename = ""):
		if filename == "":
			filename = self.Name
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(serverConfigPath, filename + ".cfg")), "r")
			self.GetFromText(f)
			self.Name = filename
			log("'" + str(filename) + "' mod config file successfully loaded")
		except:
			log("Mod config file not found for '" + str(filename))
			# If 'et_noconfig_map_behaviour' is set to 'keep' then get current config to make sure it will not be changed
			if es.getString("et_noconfig_map_behaviour") == "keep":
				log("Using current config")
				self.GetCurrent()
			else:
				log("Using default config")
		finally:
			if f is not None:
				f.close()
	def SaveToFile(self, filename = ""):
		if filename == "":
			filename = self.Name
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(serverConfigPath, filename + ".cfg")), "w")
			f.write(str(self))
			log("'" + str(filename) + "' mod config file successfully saved")
		except:
			log("Error while trying to save '" + str(filename) + "' mod config file")
		finally:
			if f is not None:
				f.close()
	def Use(self):
		es.set("et_restrict_classes", int(self.RestrictClasses))
		es.set("et_restrict_players_in_classes", int(self.RestrictPlayersInClasses))
		es.set("et_unlimited_ammo", int(self.UnlimitedAmmo))
		es.set("et_no_reload", int(self.NoReload))
		es.set("et_spawn_delay", int(self.SpawnDelay))
		es.set("et_spawn_protection_time", int(self.SpawnProtectionTime))
		es.set("et_synchronized_spawn", int(self.SynchronizedSpawn))
		es.set("et_random_classes", int(self.RandomClasses))
		es.set("et_use_colors", int(self.UseColors))
		es.set("et_round_xp", self.RoundXp)
		es.set("et_round_money", self.RoundMoney)
		es.set("et_hurt_lose_xp", int(self.HurtLoseXp))
		es.set("et_noconfig_map_behaviour", self.NoConfigMapBehaviour)
		log("'" + self.Name + "' mod config now in use")
	def __str__(self):
		return\
			"et_restrict_classes=" + str(int(self.RestrictClasses)) + "\n" +\
			"et_restrict_players_in_classes=" + str(int(self.RestrictPlayersInClasses)) + "\n" +\
			"et_unlimited_ammo=" + str(int(self.UnlimitedAmmo)) + "\n" +\
			"et_no_reload=" + str(int(self.NoReload)) + "\n" +\
			"et_spawn_delay=" + str(self.SpawnDelay) + "\n" +\
			"et_spawn_protection_time=" + str(self.SpawnProtectionTime) + "\n" +\
			"et_synchronized_spawn=" + str(int(self.SynchronizedSpawn)) + "\n" +\
			"et_random_classes=" + str(int(self.RandomClasses)) + "\n" +\
			"et_use_colors=" + str(int(self.UseColors)) + "\n" +\
			"et_round_xp=" + str(int(self.RoundXp)) + "\n" +\
			"et_round_money=" + str(int(self.RoundMoney)) + "\n" +\
			"et_hurt_lose_xp=" + str(int(self.HurtLoseXp)) + "\n" +\
			"et_noconfig_map_behaviour=" + self.NoConfigMapBehaviour
			
#___________________________________________DECLARATIONS

# --------------------- #
# DECLARATIONS #
# --------------------- #
serverConfigPath = "cstrike/addons/eventscripts/etmod_2008/server/configuration"
botQuotaModes = ["normal", "fill", "match"]
botPossibleTeams = ["any", "T", "CT"]
oldServerConfig = ServerConfig("old")

#___________________________________________FUNCTIONS

# ---------------- #
# FUNCTIONS #
# ---------------- #
''' Prepare ETmod config console commands '''
def regModVariables():
	es.set("et_restrict_classes", "1", "If set to 1, checks the stats required to join a class")
	es.set("et_restrict_players_in_classes", "1", "If set to 1, uses the restrictions defined for players quota in classes")
	es.set("et_unlimited_ammo", "0", "If set to 1, players will get ammo back once they run out of it")
	es.set("et_no_reload", "0", "If set to 1, players will not have to reloadd their weapon once their clip is empty")
	es.set("et_spawn_delay", "5", "Defines how much time (in seconds) players have to wait before respawning after dying")
	es.set("et_spawn_protection_time", "4", "Defines how much time (in seconds) players will be protected from enemies after respawning")
	es.set("et_synchronized_spawn", "0", "If set to 1, all dead players will respawn at the same time which will be based on the 'et_spawn_delay' variable")
	es.set("et_random_classes", "0", "If set to 1, players will not be able to choose their class and will be given a random one like bots")
	es.set("et_use_colors", "0", "If set to 1, the colors defined in the class configuration will be applied to players")
	es.set("et_round_xp", "0", "Defines how much xp should be given to players for winning a round")
	es.set("et_hurt_lose_xp", "0", "Defines whether players getting hurt will lose xp points")
	
#___________________________________________SERVER EVENTS

# --------------------- #
# SERVER EVENTS #
# --------------------- #
''' Script loaded '''
def load():
	# Save current server config
	oldServerConfig.GetCurrent()
	
	# Register mod variables
	regModVariables()
	
	# Log
	log("Server sub-script loaded")	
''' Script unloaded '''
def unload():
	# Reset server config as it was before loading the script
	oldServerConfig.Use()
	
	# Log
	log("Server sub-script unloaded")