#================================================================#
# SKILLS
#================================================================#

import es
import os, gamethread, random
import playerlib
from es import server_var
from math import sqrt
from etmod_2008.logs.logs import log
from etmod_2008.classes.classes import classes

#___________________________________________CLASSES

# ------------ #
# CLASSES #
# ------------ #
''' Used to represent a skill parameter '''
class SkillParameter:
	def __init__(self, type, price = 0, default = 0, add = 0, max = 0):
		self.Type = type
		self.Price = price
		self.Default = default
		self.Add = add
		self.Max = max
	def CurrentPrice(self, amount, level):
		price = sqrt(self.Price * amount * level)
		if price < self.Price: price = self.Price
		return price
	def SetValue(self, var, val):
		# Set upgrade price
		if var.endswith("price_add"):
			self.Price = int(val)
		# Default value
		elif var.endswith("default"):
			self.Default = int(val)
		# Increment value
		elif var.endswith("add"):
			self.Add = int(val)
		# Max value
		elif var.endswith("max"):
			self.Max = int(val)
	def __str__(self):
		return\
			"Type: " + str(self.Type) + "\n" +\
			"Price: " + str(self.Price) + "\n" +\
			"Default: " + str(self.Default) + "\n" +\
			"Add: " + str(self.Add) + "\n" +\
			"Max: " + str(self.Max) + "\n"
''' Used to represent a skill '''
class Skill:
	def __init__(self, name):
		self.Name = name
		self.Classes = []
		self.Level = 0
		self.CommandString = ""
		self.Price = 0
		self.Parameters = {}
		global skills
		skills[name] = self
	def CurrentPrice(self, amount, level):
		price = sqrt(self.Price * amount * level)
		if price < self.Price: price = self.Price
		return price
	def __str__(self):
		s = "Name: " + self.Name + "\nClasses: ";
		for c in self.Classes:
			s += c.Name + ", "
		s += "\nLevel: " + str(self.Level)
		s += "\nCommand string: " + self.CommandString
		s += "\nPrice: " + str(self.Price)
		s += "\nParameters:\n"
		for p in self.Parameters.values():
			s += str(p)
		s += "\n"
		return s

#___________________________________________DECLARATIONS

# --------------------- #
# DECLARATIONS #
# --------------------- #
# Possible skill parameter types
skillParameterTypes = {"duration" : 0, "intensity" : 1, "health" : 2, "armor" : 3, "speed" : 4, "distance" : 5, "primary" : 6, "secondary" : 7, "quantity" : 8}
skillParameterNames = {0 : "duration", 1 : "intensity", 2 : "health", 3 : "armor", 4 : "speed", 5 : "distance", 6 : "primary", 7 : "secondary", 8 : "quantity"}
# Skills file paths
skillsConfigPath = "cstrike/addons/eventscripts/etmod_2008/skills/configuration"
skillsDataPath = "cstrike/addons/eventscripts/etmod_2008/skills/data"
# Skills
skills = {}

#___________________________________________FUNCTIONS

# ---------------- #
# FUNCTIONS #
# ---------------- #
def loadSkills():
	f = None
	try:
		curPath = os.path.join(os.getcwd(), os.path.join(skillsConfigPath, "skills.cfg"))
		
		f = open(curPath, "r")
		skill = None
		for line in f:
			# Skip commented lines
			if line.startswith("//"):
				continue
				
			# Split based on quotes
			parts = str.split(line, "\"")
			
			# 3 parts = skill name
			if len(parts) == 3:
				log("One skill: " + parts[1])
				skill = Skill(parts[1])
				
			# 5 parts = skill property
			elif len(parts) == 5 and skill is not None:
				var = parts[1]
				val = parts[3].strip("\n")
				val = val.strip("\r")
				log("Skill info: " + var + " - Value: " + val)
				
				# Allowed classes
				if var == "classes":
					cls = val.split(";")
					for c in cls:
						if classes.has_key(c):
							skill.Classes.append(classes[c])
							
				# Required level
				elif var == "level":
					skill.Level = int(val)
					
				# Command string
				elif var == "commandstring":
					skill.CommandString = val
					
				# Price to buy
				elif var == "price_buy":
					skill.Price = int(val)
					
				# Duration parameter
				elif var.startswith("duration"):
					# Add a duration parameter if not yet done
					if not skill.Parameters.has_key("duration"):
						skill.Parameters["duration"] = SkillParameter(skillParameterTypes["duration"])
					# Update param
					skill.Parameters["duration"].SetValue(var, val)
					
				# Intensity parameter
				elif var.startswith("intensity"):
					# Add an intensity parameter if not yet done
					if not skill.Parameters.has_key("intensity"):
						skill.Parameters["intensity"] = SkillParameter(skillParameterTypes["intensity"])
					# Update param
					skill.Parameters["intensity"].SetValue(var, val)
					
				# HP parameter
				elif var.startswith("hp"):
					# Add an intensity parameter if not yet done
					if not skill.Parameters.has_key("health"):
						skill.Parameters["health"] = SkillParameter(skillParameterTypes["health"])
					# Update param
					skill.Parameters["health"].SetValue(var, val)
					
				# AP parameter
				elif var.startswith("ap"):
					# Add an armor parameter if not yet done
					if not skill.Parameters.has_key("armor"):
						skill.Parameters["armor"] = SkillParameter(skillParameterTypes["armor"])
					# Update param
					skill.Parameters["armor"].SetValue(var, val)
				# Speed parameter
				elif var.startswith("speed"):
					# Add a speed parameter if not yet done
					if not skill.Parameters.has_key("speed"):
						skill.Parameters["speed"] = SkillParameter(skillParameterTypes["speed"])
					# Update param
					skill.Parameters["speed"].SetValue(var, val)
				# Distance parameter
				elif var.startswith("distance"):
					# Add a distance parameter if not yet done
					if not skill.Parameters.has_key("distance"):
						skill.Parameters["distance"] = SkillParameter(skillParameterTypes["distance"])
					# Update param
					skill.Parameters["distance"].SetValue(var, val)
				# Primary weapon parameter
				elif var.startswith("primary"):
					# Add a primary parameter if not yet done
					if not skill.Parameters.has_key("primary"):
						skill.Parameters["primary"] = SkillParameter(skillParameterTypes["primary"])
					# Update param
					skill.Parameters["primary"].SetValue(var, val)
				# Secondary weapon parameter
				elif var.startswith("secondary"):
					# Add a secondary parameter if not yet done
					if not skill.Parameters.has_key("secondary"):
						skill.Parameters["secondary"] = SkillParameter(skillParameterTypes["secondary"])
					# Update param
					skill.Parameters["secondary"].SetValue(var, val)
				# Quantity parameter
				elif var.startswith("quantity"):
					# Add a secondary parameter if not yet done
					if not skill.Parameters.has_key("quantity"):
						skill.Parameters["quantity"] = SkillParameter(skillParameterTypes["quantity"])
					# Update param
					skill.Parameters["quantity"].SetValue(var, val)
				
		log("Skills config loaded")
	except:
		log("Error while trying to load skills config file")
	finally:
		if f is not None:
			f.close()
			
def initOwners():
	es.keygroupdelete("et_remotecharges_howners")
	es.keygroupcreate("et_remotecharges_howners")
	es.keygroupdelete("et_smokey")

def prepareNewOwner(event_var):
	id = event_var["userid"]
	sid = event_var["es_steamid"]
	
	es.give(id, "weapon_smokegrenade")
	es.keygroupdelete("et_smokey")
	es.createentitylist("smokey", "weapon_smokegrenade")
	smokeIndex = server_var["eventscripts_lastgive"]
	es.setinfo("et_smokestate", "0")
	es.setinfo("et_smoketeam", "0")
	es.setinfo("et_smokeowner", "0")
	es.getindexprop("et_smoketeam", smokeIndex, "CBaseEntity.m_iTeam")
	if int(server_var["et_smoketeam"]) < 2:
		es.getindexprop("et_smokeowner", smokeIndex, "CSmokeGrenade.baseclass.baseclass.baseclass.m_hOwner")
		owner = str(server_var["et_smokeowner"])
		owner = owner.replace("1", "!")
		owner = owner.replace("2", "@")
		owner = owner.replace("3", "#")
		owner = owner.replace("4", "$")
		owner = owner.replace("5", "r")
		owner = owner.replace("6", "^")
		owner = owner.replace("7", "&")
		owner = owner.replace("8", "*")
		owner = owner.replace("9", "(")
		owner = owner.replace("0", ")")
		owner += " "
		es.keycreate("et_remotecharges_howners", sid)
		es.keysetvalue("et_remotecharges_howners", sid, "howner", owner)
		es.give(id, "player_weaponstrip")
		es.fire(id, "player_weaponstrip Strip")
		
def detonateCharge(event_var):
	id = event_var["userid"]
	sid = event_var["es_steamid"]
	x = event_var["x"]
	y = event_var["y"]
	z = event_var["z"]
	
	es.remove("env_explosion")
	es.give(id, "env_explosion")
	es.setinfo("et_owner", "0")
	es.keygetvalue("et_owner", "et_remotecharges_howners", id, "howner")
	es.keygroupdelete("et_explosions")
	es.createentitylist("et_explosions", "env_explosion")
	if int(server_var["et_owner"]) != 0:
		owner = str(server_var["et_owner"])
		owner = owner.replace("1", "!")
		owner = owner.replace("2", "@")
		owner = owner.replace("3", "#")
		owner = owner.replace("4", "$")
		owner = owner.replace("5", "r")
		owner = owner.replace("6", "^")
		owner = owner.replace("7", "&")
		owner = owner.replace("8", "*")
		owner = owner.replace("9", "(")
		owner = owner.replace("0", ")")
		es.server.queuecmd('es_foreachkey et_explosion in et_explosions "esnq es_setindexprop server_var(et_explosion) CBaseEntity.m_hOwnerEntity ' + owner + '"')
	es.server.queuecmd('es_xfire ' + id + ' env_explosion addoutput "imagnitude 200"')
	es.server.queuecmd('es_xfire ' + id + ' env_explosion addoutput "angles -90 0 0"')
	es.server.queuecmd('es_xfire ' + id + ' env_explosion addoutput "origin ' + x + ' ' + y + ' ' + z + '"')
	es.server.queuecmd('es_xfire ' + id + ' env_explosion DispatchEffect Smoke')
	es.server.queuecmd('es_xfire ' + id + ' env_explosion Explode')

def expansiveExplodingBullet(event_var):
	# Get info
	id = event_var["userid"]
	x = event_var["x"]
	y = event_var["y"]
	z = event_var["z"]
	
	es.keygroupdelete("remoteCharges_howner")
	es.keygroupcreate("remoteCharges_howner")
	es.give(id, "weapon_smokegrenade")
	es.keygroupdelete("smokey")
	es.createentitylist("smokey", "weapon_smokegrenade")
	es.setinfo("smokeindex", server_var["eventscripts_lastgive"])
	
	es.setinfo("smokestate", "0")
	es.setinfo("smoketeam", "0")
	es.setinfo("smokeowner", "0")
	es.getindexprop("smoketeam", server_var["smokeindex"], "CBaseEntity.m_iTeam")
	if int(server_var["smoketeam"]) < 2:
		es.getindexprop("smokeowner", server_var["smokeindex"], "CSmokeGrenade.baseclass.baseclass.baseclass.m_hOwner")
		owner = str(server_var["smokeowner"])
		owner = owner.replace("1", "!")
		owner = owner.replace("2", "@")
		owner = owner.replace("3", "#")
		owner = owner.replace("4", "$")
		owner = owner.replace("5", "r")
		owner = owner.replace("6", "^")
		owner = owner.replace("7", "&")
		owner = owner.replace("8", "*")
		owner = owner.replace("9", "(")
		owner = owner.replace("0", ")")
		owner = owner + " "
		es.setinfo("smokeowner", owner)
		es.server.queuecmd("es_keycreate remoteCharges_howner event_var(userid)")#
		es.server.queuecmd("es_keysetvalue remoteCharges_howner event_var(userid) howner server_var(smokeowner)")#
		es.delayed(1, "es_give event_var(userid) player_weaponstrip")
		es.delayed(2, "es_fire event_var(userid) player_weaponstrip Strip")
	es.server.queuecmd("es_remove env_explosion")
	es.server.queuecmd("es_give event_var(userid) env_explosion")
	es.server.queuecmd("es_setinfo remoteCharges_owner 0")
	es.server.queuecmd("es_keygetvalue remoteCharges_owner remoteCharges_howner event_var(userid) howner")#
	es.server.queuecmd("es_keygroupdelete remoteCharges_explosions")
	es.server.queuecmd("es_createentitylist remoteCharges_explosions env_explosion")
	if int(server_var["remoteCharges_owner"]) != 0:
		owner = str(server_var["remoteCharges_owner"])
		owner = owner.replace("1", "!")
		owner = owner.replace("2", "@")
		owner = owner.replace("3", "#")
		owner = owner.replace("4", "$")
		owner = owner.replace("5", "r")
		owner = owner.replace("6", "^")
		owner = owner.replace("7", "&")
		owner = owner.replace("8", "*")
		owner = owner.replace("9", "(")
		owner = owner.replace("0", ")")
		es.setinfo("remoteCharges_owner", owner)
		es.server.queuecmd('es_foreachkey remoteCharges_exp in remoteCharges_explosions "esnq es_xsetindexprop server_var(remoteCharges_exp) CBaseEntity.m_hOwnerEntity server_var(remoteCharges_owner)"')
	es.server.queuecmd("es_xsetinfo RCoutput 0")
	es.server.queuecmd('es_format RCoutput "imagnitude %1" 250')
	es.server.queuecmd("es_fire event_var(userid) env_explosion addoutput server_var(RCoutput)")
	es.server.queuecmd('es_format RCoutput "angles -90 0 0"')
	es.server.queuecmd("es_fire event_var(userid) env_explosion addoutput server_var(RCoutput)")
	es.server.queuecmd("es_xsetinfo RCx 0")
	es.server.queuecmd("es_xsetinfo RCy 0")
	es.server.queuecmd("es_xsetinfo RCz 0")
	es.server.queuecmd("es est_GetViewCoord event_var(userid) RCx RCy RCz")
	es.server.queuecmd('es_format RCoutput "origin %1 %2 %3" server_var(RCx) server_var(RCy) server_var(RCz)')
	es.server.queuecmd("es_fire event_var(userid) env_explosion addoutput server_var(RCoutput)")
	es.server.queuecmd("es_fire event_var(userid) env_explosion DispatchEffect Smoke")
	es.server.queuecmd("es_fire event_var(userid) env_explosion Explode")
	
def cheapExplodingBullet(event_var):
	# Get info
	id = event_var["userid"]
	x = event_var["x"]
	y = event_var["y"]
	z = event_var["z"]
	
	if not playerlib.getPlayer(id).attributes["isbot"]:
		es.server.queuecmd("es_xremove env_explosion")
		es.server.queuecmd("es_xgive " + id + " env_explosion")
		es.server.queuecmd('es_xfire ' + id + ' env_explosion addoutput "imagnitude 200"')
		es.server.queuecmd('es_xfire ' + id + ' env_explosion addoutput "angles -90 0 0"')
		es.server.queuecmd('es_xfire ' + id + ' env_explosion addoutput "origin ' + x + ' ' + y + ' ' + z + '"')
		es.server.queuecmd('es_xfire ' + id + ' env_explosion DispatchEffect Smoke')
		es.server.queuecmd('es_xfire ' + id + ' env_explosion Explode')
	es.setinfo("RCplayer", "0")
	es.server.queuecmd('es est_NearCoord RCplayer #all 500 ' + x + ' ' + y + ' ' + z + ' "es est_Fade server_var(RCplayer) 0 5 1 255 255 255 255"')
	#es.server.queuecmd('es est_NearCoord RCplayer #all 500 ' + x + ' ' + y + ' ' + z + ' "es_cexec server_var(RCplayer) r_screenoverlay Effects/flashbang;es_delayed 5 es_cexec server_var(RCplayer) r_screenoverlay 0"')

''' Trigger an explosion '''
def createExplosion(id, intensity, loc):
	# Effect
	es.server.queuecmd("es_xremove env_explosion")
	es.server.queuecmd("es_xgive " + id + " env_explosion")
	es.server.queuecmd('es_xfire ' + id + ' env_explosion addoutput "imagnitude ' + str(intensity) + '"')
	es.server.queuecmd('es_xfire ' + id + ' env_explosion addoutput "angles -90 0 0"')
	es.server.queuecmd('es_xfire ' + id + ' env_explosion addoutput "origin ' + str(loc[0]) + ' ' + str(loc[1]) + ' ' + str(loc[2]) + '"')
	es.server.queuecmd('es_xfire ' + id + ' env_explosion DispatchEffect Smoke')
	es.server.queuecmd('es_xfire ' + id + ' env_explosion Explode')
	
	# Sound
	es.server.queuecmd('es_xemitsound entity ' + str(server_var["eventscripts_lastgive"]) + ' "ambient/explosions/explode_%(rnd)s.wav" 1.0 0.2'\
		% {"rnd" : random.randint(1, 9)})
	
''' Chained explosions '''
def chainedExplosions(id, intensity, locs, delay):
	# If we ever get here without any loc, lets leave
	if len(locs) < 1:
		return
		
	# Prepare a variable so that killed player can reward the charge owner
	es.set("et_charge_owner_hurt", id)
	es.set("et_charge_owner_kill", id)
	
	# Get current bomb location
	loc = locs[0]
	
	# Trigger the explosion
	createExplosion(id, intensity, loc)
	
	# Loop
	del locs[0]
	if len(locs) > 0:
		gamethread.delayed(delay, chainedExplosions, (id, intensity, locs, delay))
	else:
		es.tell(id, "Sticky bombs destroyed")
	
#___________________________________________SERVER EVENTS

# --------------------- #
# SERVER EVENTS #
# --------------------- #
''' Script loaded '''
def load():
	# Load skills
	loadSkills()
	
	# Log
	log("Skills sub-script loaded")	
''' Script unloaded '''
def unload():
	# Log
	log("Skills sub-script unloaded")
	