#================================================================#
# STATS
#================================================================#

import es
import os
import pickle
from math import sqrt
from etmod_2008.logs.logs import *
from etmod_2008.classes.classes import *

#___________________________________________CLASSES

# ------------ #
# CLASSES #
# ------------ #
''' Used to represent an ETmod player stats  '''
class Stats:
	def __init__(self):
		self.Money = 0
		self.SavedMoney = 0
		self.Classes = {}
		self.Classes["noclass"] = {}
	def GetLevel(self, classname):
		if not classes.has_key(classname):
			return 0
		if not self.Classes.has_key(classname):
			self.Classes[classname] = {}
		if not self.Classes[classname].has_key("xp"):
			self.Classes[classname]["xp"] = 0
		return int(pow(float(self.Classes[classname]["xp"]), 0.65) / 10)
	def Update(self, attackerClass, victimClass, action, count = 1, ratio = 1):
		# If that class does not exist, return
		if not classes.has_key(attackerClass) or not classes.has_key(victimClass):
			return
			
		# Create class key if it doesn't yet exist
		if not self.Classes.has_key(attackerClass):
			self.Classes[attackerClass] = {}
		if not self.Classes.has_key(victimClass):
			self.Classes[victimClass] = {}
			
		# Create info if it doesn't yet exist for that class
		if not self.Classes[attackerClass].has_key(action):
			self.Classes[attackerClass][action] = 0
		if not self.Classes[victimClass].has_key(action):
			self.Classes[victimClass][action] = 0
			
		# Create xp key if it does not yet exist
		if not self.Classes[attackerClass].has_key("xp"):
			self.Classes[attackerClass]["xp"] = 0
		if not self.Classes[victimClass].has_key("xp"):
			self.Classes[victimClass]["xp"] = 0
			
		# Minimum count = 1
		if count < 1: count = 1
			
		# Minimum ratio = 1 (it's usually the level of the attacker, or the number of players for round_end, ...)
		if ratio < 1: ratio = 1
		
		# How the ratio will be used in the calculation
		ratio = sqrt(ratio)
		
	#try:
		# Update stats info
		self.Classes[attackerClass][action] += count
			
		# Add xp
		if action == "kills":
			self.Money += int(ratio * classes[victimClass].Actions.KillMoney * count)
			self.Classes[attackerClass]["xp"] += int(ratio * classes[victimClass].Actions.KillXp * count)
		elif action == "deaths":
			self.Money -= int(ratio * classes[attackerClass].Actions.DeathMoney * count)
			self.Classes[victimClass]["xp"] -= int(ratio * classes[attackerClass].Actions.DeathXp * count)
		elif action == "suicides":
			self.Money -= int(ratio * classes[victimClass].Actions.SuicideMoney * count)
			self.Classes[attackerClass]["xp"] -= int(ratio * classes[victimClass].Actions.SuicideXp * count)
		elif action == "teamkills":
			self.Money += int(ratio * classes[attackerClass].Actions.TeamKillMoney * count)
			self.Classes[victimClass]["xp"] += int(ratio * classes[attackerClass].Actions.TeamKillXp * count)
		elif action == "teamkillsdone":
			self.Money -= int(ratio * classes[victimClass].Actions.TeamKillMoney * count)
			self.Classes[attackerClass]["xp"] -= int(ratio * classes[victimClass].Actions.TeamKillXp * count)
		elif action == "headshots":
			self.Money += int(ratio * classes[victimClass].Actions.HeadshotMoney * count)
			self.Classes[attackerClass]["xp"] += int(ratio * classes[victimClass].Actions.HeadshotXp * count)
		elif action == "knives":
			self.Money += int(ratio * classes[victimClass].Actions.KnifeMoney * count)
			self.Classes[attackerClass]["xp"] += int(ratio * classes[victimClass].Actions.KnifeXp * count)
		elif action == "hurts":
			self.Money -= int(ratio * classes[attackerClass].Actions.HurtMoney * count)
			hurtLoseXp = (es.getInt("et_hurt_lose_xp") == 1)
			if hurtLoseXp:
				self.Classes[victimClass]["xp"] -= int(ratio * classes[attackerClass].Actions.HurtXp * count)
		elif action == "hurtsdone":
			self.Money += int(ratio * classes[victimClass].Actions.HurtMoney * count)
			self.Classes[attackerClass]["xp"] += int(ratio * classes[victimClass].Actions.HurtXp * count)
		elif action == "selfhurts":
			self.Money -= int(ratio * classes[victimClass].Actions.SelfHurtMoney * count)
			self.Classes[attackerClass]["xp"] -= int(ratio * classes[victimClass].Actions.SelfHurtXp * count)
		elif action == "teamhurts":
			self.Money += int(ratio * classes[attackerClass].Actions.TeamHurtMoney * count)
			self.Classes[victimClass]["xp"] += int(ratio * classes[attackerClass].Actions.TeamHurtXp * count)
		elif action == "teamhurtsdone":
			self.Money -= int(ratio * classes[victimClass].Actions.TeamHurtMoney * count)
			self.Classes[attackerClass]["xp"] -= int(ratio * classes[victimClass].Actions.TeamHurtXp * count)
		elif action == "heals":
			self.Money += int(ratio * classes[victimClass].Actions.HealMoney * count)
			self.Classes[attackerClass]["xp"] += int(ratio * classes[victimClass].Actions.HealXp * count)
		elif action == "plants":
			self.Money += int(ratio * classes[victimClass].Actions.PlantMoney * count)
			self.Classes[attackerClass]["xp"] += int(ratio * classes[victimClass].Actions.PlantXp * count)
		elif action == "defuses":
			self.Money += int(ratio * classes[victimClass].Actions.DefuseMoney * count)
			self.Classes[attackerClass]["xp"] += int(ratio * classes[victimClass].Actions.DefuseXp * count)
		elif action == "explodes":
			self.Money += int(ratio * classes[victimClass].Actions.ExplodeMoney * count)
			self.Classes[attackerClass]["xp"] += int(ratio * classes[victimClass].Actions.ExplodeXp * count)
		elif action == "supplies":
			self.Money += int(ratio * classes[victimClass].Actions.SupplyMoney * count)
			self.Classes[attackerClass]["xp"] += int(ratio * classes[victimClass].Actions.SupplyXp * count)
		elif action == "rescues":
			self.Money += int(ratio * classes[victimClass].Actions.RescueMoney * count)
			self.Classes[attackerClass]["xp"] += int(ratio * classes[victimClass].Actions.RescueXp * count)
		elif action == "hosthurts":
			self.Money -= int(ratio * classes[victimClass].Actions.TeamHurtMoney * count)
			self.Classes[attackerClass]["xp"] -= int(ratio * classes[victimClass].Actions.TeamHurtXp * count)
		elif action == "hostkills":
			self.Money -=  int(ratio * classes[victimClass].Actions.TeamKillMoney * count)
			self.Classes[attackerClass]["xp"] -=  int(ratio * classes[victimClass].Actions.TeamKillXp * count)
		elif action == "roundswon":
			self.Money += int(ratio * es.getInt("et_round_money") * count)
			self.Classes[attackerClass]["xp"] += int(ratio * es.getInt("et_round_xp") * count)
			
		# Set to 0 if below
		if self.Money < 0: self.Money = 0
		if self.Classes[attackerClass]["xp"] < 0: self.Classes[attackerClass]["xp"] = 0
		if self.Classes[victimClass]["xp"] < 0: self.Classes[victimClass]["xp"] = 0
	# except:
		# log("Error while updating player stats for " + str(action) +\
			# " in class " + str(attackerClass) + " with action " + str(action) +\
			# " and quantity " + str(count))
	def Save(self, filename, playerName = ""):
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(statsDataPath, filename + ".st")), "w")
			#pickle.dump(self, f)
			if playerName != "":
				f.write("# " + playerName + "'s stats file\n\n")
			f.write("money=" + str(self.Money) + "\n")
			f.write("saved-money=" + str(self.SavedMoney) + "\n")
			for c in self.Classes.keys():
				f.write("[" + str(c) + "]\n")
				for k in self.Classes[c].keys():
					f.write(str(k) + "=" + str(self.Classes[c][k]) + "\n")
			log("'" + str(filename) + "' stats file successfully saved")
		except:
			log("Error while trying to save '" + str(filename) + "' stats file")
		finally:
			if f is not None:
				f.close()
	def Load(self, filename):
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(statsDataPath, filename + ".st")), "r")
			#other = pickle.load(f)
			#self.Classes = other.Classes.copy()
			#self.Money = other.Money
			#self.Classes = pickle.load(f).Classes
			self.__init__()
			cl = ""
			for line in f:
				line = line.strip("\n")
				line = line.strip("\r")
				if line.startswith("[") and line.endswith("]"):
					cl = line.strip("[]")
					if not self.Classes.has_key(cl):
						self.Classes[cl] = {}
				else:
					parts = line.split("=")
					if len(parts) == 2:
						parts[1] = parts[1].strip("\n")
						parts[1] = parts[1].strip("\r")
						if parts[0] == "money":
							self.Money = int(parts[1])
						elif parts[0] == "saved-money":
							self.SavedMoney = int(parts[1])
						elif cl != "":
							self.Classes[cl][parts[0]] = int(parts[1])
						
			log("'" + str(filename) + "' stats file successfully loaded")
		except:
			log("Error while trying to load '" + str(filename) + "' stats file")
		finally:
			if f is not None:
				f.close()
#___________________________________________DECLARATIONS

# --------------------- #
# DECLARATIONS #
# --------------------- #
statsDataPath = "cstrike/addons/eventscripts/etmod_2008/stats/data"

#___________________________________________SERVER EVENTS

# --------------------- #
# SERVER EVENTS #
# --------------------- #
''' Script loaded '''
def load():
	log("Stats sub-script loaded")	
''' Script unloaded '''
def unload():
	log("Stats sub-script unloaded")